@@ -54,7 +54,7 @@ NSRect PRectangleToNSRect(PRectangle& rc)
  */
 PRectangle NSRectToPRectangle(NSRect& rc)
 {
-  return PRectangle(rc.origin.x, rc.origin.y,
+  return PRectangle(static_cast<XYPOSITION>(rc.origin.x), static_cast<XYPOSITION>(rc.origin.y),
 					static_cast<XYPOSITION>(NSMaxX(rc)),
 					static_cast<XYPOSITION>(NSMaxY(rc)));
 }
@@ -223,7 +223,7 @@ void SurfaceImpl::Init(SurfaceID sid, WindowID)
 
 //--------------------------------------------------------------------------------------------------
 
-void SurfaceImpl::InitPixMap(int width, int height, Surface* /* surface_ */, WindowID /* wid */)
+void SurfaceImpl::InitPixMap(int width, int height, Surface* surface_, WindowID /* wid */)
 {
   Release();
 
@@ -269,6 +269,18 @@ void SurfaceImpl::InitPixMap(int width, int height, Surface* /* surface_ */, Win
     CGContextSetRGBFillColor( gc, 1.0, 1.0, 1.0, 1.0 );
     CGContextFillRect( gc, CGRectMake( 0, 0, width, height ) );
   }
+
+  if (surface_)
+  {
+    SurfaceImpl *psurfOther = static_cast<SurfaceImpl *>(surface_);
+    unicodeMode = psurfOther->unicodeMode;
+    codePage = psurfOther->codePage;
+  }
+  else
+  {
+    unicodeMode = true;
+    codePage = SC_CP_UTF8;
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -541,8 +553,7 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc, ColourDesired fore, ColourDesi
 
   // Create a rectangle with semicircles at the corners
   const int MAX_RADIUS = 4;
-  int radius = Platform::Minimum( MAX_RADIUS, rc.Height()/2 );
-  radius = Platform::Minimum( radius, rc.Width()/2 );
+  const int radius = std::min(MAX_RADIUS, static_cast<int>(std::min(rc.Height()/2, rc.Width()/2)));
 
   // Points go clockwise, starting from just below the top left
   // Corners are kept together, so we can easily create arcs to connect them
@@ -784,10 +795,10 @@ void SurfaceImpl::CopyImageRectangle(Surface &surfaceSource, PRectangle srcRect,
   CGRect drawRect = CGRectMake (0, 0, w, h);
   if (!CGRectEqualToRect (src, dst))
   {
-    float sx = CGRectGetWidth(dst) / CGRectGetWidth(src);
-    float sy = CGRectGetHeight(dst) / CGRectGetHeight(src);
-    float dx = CGRectGetMinX(dst) - (CGRectGetMinX(src) * sx);
-    float dy = CGRectGetMinY(dst) - (CGRectGetMinY(src) * sy);
+    CGFloat sx = CGRectGetWidth(dst) / CGRectGetWidth(src);
+    CGFloat sy = CGRectGetHeight(dst) / CGRectGetHeight(src);
+    CGFloat dx = CGRectGetMinX(dst) - (CGRectGetMinX(src) * sx);
+    CGFloat dy = CGRectGetMinY(dst) - (CGRectGetMinY(src) * sy);
     drawRect = CGRectMake (dx, dy, w*sx, h*sy);
   }
   CGContextSaveGState (gc);
@@ -959,11 +970,11 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, XYPOSITION
 			size_t codeUnits = (lenChar < 4) ? 1 : 2;
 			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, ui+codeUnits, NULL);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
-				positions[i++] = xPosition;
+				positions[i++] = static_cast<XYPOSITION>(xPosition);
 			}
 			ui += codeUnits;
 		}
-		int lastPos = 0;
+		XYPOSITION lastPos = 0.0f;
 		if (i > 0)
 			lastPos = positions[i-1];
 		while (i<len) {
@@ -975,14 +986,14 @@ void SurfaceImpl::MeasureWidths(Font &font_, const char *s, int len, XYPOSITION
 			size_t lenChar = Platform::IsDBCSLeadByte(codePage, s[i]) ? 2 : 1;
 			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, ui+1, NULL);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
-				positions[i++] = xPosition;
+				positions[i++] = static_cast<XYPOSITION>(xPosition);
 			}
 			ui++;
 		}
 	} else {	// Single byte encoding
 		for (int i=0;i<len;i++) {
 			CGFloat xPosition = CTLineGetOffsetForStringIndex(mLine, i+1, NULL);
-			positions[i] = xPosition;
+			positions[i] = static_cast<XYPOSITION>(xPosition);
 		}
 	}
 
@@ -994,7 +1005,7 @@ XYPOSITION SurfaceImpl::WidthText(Font &font_, const char *s, int len) {
     CFStringEncoding encoding = EncodingFromCharacterSet(unicodeMode, FontCharacterSet(font_));
     textLayout->setText (reinterpret_cast<const UInt8*>(s), len, encoding, *reinterpret_cast<QuartzTextStyle*>(font_.GetID()));
 
-	return textLayout->MeasureStringWidth();
+	return static_cast<XYPOSITION>(textLayout->MeasureStringWidth());
   }
   return 1;
 }
@@ -1021,7 +1032,7 @@ XYPOSITION SurfaceImpl::Ascent(Font &font_) {
     return 1;
 
 	float ascent = reinterpret_cast<QuartzTextStyle*>( font_.GetID() )->getAscent();
-	return ascent + 0.5;
+	return ascent + 0.5f;
 
 }
 
@@ -1030,7 +1041,7 @@ XYPOSITION SurfaceImpl::Descent(Font &font_) {
     return 1;
 
 	float descent = reinterpret_cast<QuartzTextStyle*>( font_.GetID() )->getDescent();
-	return descent + 0.5;
+	return descent + 0.5f;
 
 }
 
@@ -1043,14 +1054,13 @@ XYPOSITION SurfaceImpl::ExternalLeading(Font &font_) {
     return 1;
 
 	float leading = reinterpret_cast<QuartzTextStyle*>( font_.GetID() )->getLeading();
-	return leading + 0.5;
+	return leading + 0.5f;
 
 }
 
 XYPOSITION SurfaceImpl::Height(Font &font_) {
 
-	int ht = Ascent(font_) + Descent(font_);
-	return ht;
+	return Ascent(font_) + Descent(font_);
 }
 
 XYPOSITION SurfaceImpl::AverageCharWidth(Font &font_) {
@@ -1059,7 +1069,7 @@ XYPOSITION SurfaceImpl::AverageCharWidth(Font &font_) {
     return 1;
 
   const int sizeStringLength = ELEMENTS( sizeString );
-  int width = WidthText( font_, sizeString, sizeStringLength  );
+  XYPOSITION width = WidthText( font_, sizeString, sizeStringLength  );
 
   return (int) ((width / (float) sizeStringLength) + 0.5);
 }
@@ -1096,21 +1106,7 @@ Window::~Window()
 {
 }
 
-//--------------------------------------------------------------------------------------------------
-
-void Window::Destroy()
-{
-  if (wid)
-  {
-    id idWin = reinterpret_cast<id>(wid);
-    if ([idWin isKindOfClass: [NSWindow class]])
-    {
-      NSWindow* win = reinterpret_cast<NSWindow*>(idWin);
-      [win release];
-    }
-  }
-  wid = 0;
-}
+// Window::Destroy needs to see definition of ListBoxImpl so is located after ListBoxImpl
 
 //--------------------------------------------------------------------------------------------------
 
@@ -1122,15 +1118,13 @@ bool Window::HasFocus()
 
 //--------------------------------------------------------------------------------------------------
 
-static int ScreenMax(NSWindow* win)
+static CGFloat ScreenMax(NSWindow* win)
 {
-  NSScreen* screen = [win screen];
-  if (!screen)
-    screen = [NSScreen mainScreen];
-  NSRect frame = [screen frame];
-  return frame.origin.y + frame.size.height;
+  return NSMaxY([[NSScreen mainScreen] frame]);
 }
 
+//--------------------------------------------------------------------------------------------------
+
 PRectangle Window::GetPosition()
 {
   if (wid)
@@ -1144,7 +1138,7 @@ PRectangle Window::GetPosition()
       NSView* view = reinterpret_cast<NSView*>(idWin);
       win = [view window];
       rect = [view convertRect: [view bounds] toView: nil];
-      rect.origin = [win convertBaseToScreen:rect.origin];
+      rect = [win convertRectToScreen:rect];
     }
     else
     {
@@ -1155,8 +1149,8 @@ PRectangle Window::GetPosition()
     CGFloat screenHeight = ScreenMax(win);
     // Invert screen positions to match Scintilla
     return PRectangle(
-        NSMinX(rect), static_cast<XYPOSITION>(screenHeight - NSMaxY(rect)),
-        NSMaxX(rect), static_cast<XYPOSITION>(screenHeight - NSMinY(rect)));
+        static_cast<XYPOSITION>(NSMinX(rect)), static_cast<XYPOSITION>(screenHeight - NSMaxY(rect)),
+        static_cast<XYPOSITION>(NSMaxX(rect)), static_cast<XYPOSITION>(screenHeight - NSMinY(rect)));
   }
   else
   {
@@ -1177,7 +1171,7 @@ void Window::SetPosition(PRectangle rc)
       // Moves this view inside the parent view
       NSRect nsrc = NSMakeRect(rc.left, rc.bottom, rc.Width(), rc.Height());
       NSView* view = reinterpret_cast<NSView*>(idWin);
-      nsrc.origin = [[view window] convertScreenToBase:nsrc.origin];
+      nsrc = [[view window] convertRectFromScreen:nsrc];
       [view setFrame: nsrc];
     }
     else
@@ -1185,7 +1179,7 @@ void Window::SetPosition(PRectangle rc)
       // NSWindow
       PLATFORM_ASSERT([idWin isKindOfClass: [NSWindow class]]);
       NSWindow* win = reinterpret_cast<NSWindow*>(idWin);
-      int screenHeight = ScreenMax(win);
+      CGFloat screenHeight = ScreenMax(win);
       NSRect nsrc = NSMakeRect(rc.left, screenHeight - rc.bottom,
           rc.Width(), rc.Height());
       [win setFrame: nsrc display:YES];
@@ -1335,16 +1329,28 @@ PRectangle Window::GetMonitorRect(Point)
   if (wid)
   {
     id idWin = reinterpret_cast<id>(wid);
+    if ([idWin isKindOfClass: [NSView class]])
+    {
+      NSView* view = reinterpret_cast<NSView*>(idWin);
+      idWin = [view window];
+    }
     if ([idWin isKindOfClass: [NSWindow class]])
     {
+      PRectangle rcPosition = GetPosition();
+
       NSWindow* win = reinterpret_cast<NSWindow*>(idWin);
       NSScreen* screen = [win screen];
-      NSRect rect = [screen frame];
+      NSRect rect = [screen visibleFrame];
       CGFloat screenHeight = rect.origin.y + rect.size.height;
       // Invert screen positions to match Scintilla
-      return PRectangle(
-          NSMinX(rect), static_cast<XYPOSITION>(screenHeight - NSMaxY(rect)),
-          NSMaxX(rect), static_cast<XYPOSITION>(screenHeight - NSMinY(rect)));
+      PRectangle rcWork(
+          static_cast<XYPOSITION>(NSMinX(rect)), static_cast<XYPOSITION>(screenHeight - NSMaxY(rect)),
+          static_cast<XYPOSITION>(NSMaxX(rect)), static_cast<XYPOSITION>(screenHeight - NSMinY(rect)));
+      PRectangle rcMonitor(rcWork.left - rcPosition.left,
+                           rcWork.top - rcPosition.top,
+                           rcWork.right - rcPosition.left,
+                           rcWork.bottom - rcPosition.top);
+      return rcMonitor;
     }
   }
   return PRectangle();
@@ -1383,6 +1389,8 @@ static NSImage* ImageFromXPM(XPM* pxpm)
 
 //----------------- ListBox and related classes ----------------------------------------------------
 
+//----------------- IListBox -----------------------------------------------------------------------
+
 namespace {
 
 // unnamed namespace hides IListBox interface
@@ -1527,9 +1535,9 @@ private:
   int lineHeight;
   bool unicodeMode;
   int desiredVisibleRows;
-  unsigned int maxItemWidth;
+  XYPOSITION maxItemWidth;
   unsigned int aveCharWidth;
-  unsigned int maxIconWidth;
+  XYPOSITION maxIconWidth;
   Font font;
   int maxWidth;
 
@@ -1544,9 +1552,21 @@ private:
   void* doubleClickActionData;
 
 public:
-  ListBoxImpl() : lineHeight(10), unicodeMode(false),
-    desiredVisibleRows(5), maxItemWidth(0), aveCharWidth(8), maxIconWidth(0),
-    doubleClickAction(NULL), doubleClickActionData(NULL)
+  ListBoxImpl() :
+    lineHeight(10),
+    unicodeMode(false),
+    desiredVisibleRows(5),
+    maxItemWidth(0),
+    aveCharWidth(8),
+    maxIconWidth(0),
+    maxWidth(2000),
+    table(nil),
+    scroller(nil),
+    colIcon(nil),
+    colText(nil),
+    ds(nil),
+    doubleClickAction(nullptr),
+    doubleClickActionData(nullptr)
   {
   }
   ~ListBoxImpl() {}
@@ -1576,6 +1596,9 @@ public:
   }
   void SetList(const char* list, char separator, char typesep);
 
+  // To clean up when closed
+  void ReleaseViews();
+
   // For access from AutoCompletionDataSource implement IListBox
   int Rows();
   NSImage* ImageForRow(NSInteger row);
@@ -1623,6 +1646,7 @@ void ListBoxImpl::Create(Window& /*parent*/, int /*ctrlID*/, Scintilla::Point pt
 
   [table setTarget:ds];
   [table setDoubleAction:@selector(doubleClick:)];
+  table.selectionHighlightStyle = NSTableViewSelectionHighlightStyleSourceList;
   wid = winLB;
 }
 
@@ -1660,19 +1684,20 @@ PRectangle ListBoxImpl::GetDesiredRect()
   rcDesired = GetPosition();
 
   // There appears to be an extra pixel above and below the row contents
-  int itemHeight = [table rowHeight] + 2;
+  CGFloat itemHeight = [table rowHeight] + 2;
 
   int rows = Length();
   if ((rows == 0) || (rows > desiredVisibleRows))
     rows = desiredVisibleRows;
 
-  rcDesired.bottom = rcDesired.top + itemHeight * rows;
+  rcDesired.bottom = rcDesired.top + static_cast<XYPOSITION>(itemHeight * rows);
   rcDesired.right = rcDesired.left + maxItemWidth + aveCharWidth;
 
   if (Length() > rows)
   {
     [scroller setHasVerticalScroller:YES];
-    rcDesired.right += [NSScroller scrollerWidth];
+    rcDesired.right += [NSScroller scrollerWidthForControlSize:NSRegularControlSize
+						 scrollerStyle:NSScrollerStyleLegacy];
   }
   else
   {
@@ -1689,7 +1714,21 @@ int ListBoxImpl::CaretFromEdge()
   if ([colIcon isHidden])
     return 3;
   else
-    return 6 + [colIcon width];
+    return 6 + static_cast<int>([colIcon width]);
+}
+
+void ListBoxImpl::ReleaseViews()
+{
+  [table release];
+  table = nil;
+  [scroller release];
+  scroller = nil;
+  [colIcon release];
+  colIcon = nil;
+  [colText release ];
+  colText = nil;
+  [ds release];
+  ds = nil;
 }
 
 void ListBoxImpl::Clear()
@@ -1705,7 +1744,7 @@ void ListBoxImpl::Append(char* s, int type)
   ld.Add(count, type, s);
 
   Scintilla::SurfaceImpl surface;
-  unsigned int width = surface.WidthText(font, s, static_cast<int>(strlen(s)));
+  XYPOSITION width = surface.WidthText(font, s, static_cast<int>(strlen(s)));
   if (width > maxItemWidth)
   {
     maxItemWidth = width;
@@ -1717,7 +1756,7 @@ void ListBoxImpl::Append(char* s, int type)
     NSImage* img = it->second;
     if (img)
     {
-      unsigned int widthIcon = img.size.width;
+      XYPOSITION widthIcon = static_cast<XYPOSITION>(img.size.width);
       if (widthIcon > maxIconWidth)
       {
         [colIcon setHidden: NO];
@@ -1907,6 +1946,28 @@ ListBox* ListBox::Allocate()
 	return lb;
 }
 
+//--------------------------------------------------------------------------------------------------
+
+void Window::Destroy()
+{
+  ListBoxImpl *listbox = dynamic_cast<ListBoxImpl *>(this);
+  if (listbox)
+  {
+    listbox->ReleaseViews();
+  }
+  if (wid)
+  {
+    id idWin = reinterpret_cast<id>(wid);
+    if ([idWin isKindOfClass: [NSWindow class]])
+    {
+      NSWindow* win = reinterpret_cast<NSWindow*>(idWin);
+      [win release];
+    }
+  }
+  wid = 0;
+}
+
+
 //----------------- ScintillaContextMenu -----------------------------------------------------------
 
 @implementation ScintillaContextMenu : NSMenu
@@ -2030,7 +2091,7 @@ int Platform::DefaultFontSize()
  * Returns the time span in which two consecutive mouse clicks must occur to be considered as
  * double click.
  *
- * @return
+ * @return time span in milliseconds
  */
 unsigned int Platform::DoubleClickTime()
 {
