@@ -75,13 +75,7 @@ void WEntry::SetText(const GUI::gui_char *text) {
 
 void WEntry::SetValid(GtkEntry *entry, bool valid) {
 #if GTK_CHECK_VERSION(3,0,0)
-	if (valid) {
-		GdkRGBA black = { 0, 0, 0, 1};
-		gtk_widget_override_color(GTK_WIDGET(entry), (GtkStateFlags)GTK_STATE_NORMAL, &black);
-	} else {
-		GdkRGBA red = { 1.0, 0, 0, 1};
-		gtk_widget_override_color(GTK_WIDGET(entry), (GtkStateFlags)GTK_STATE_NORMAL, &red);
-	}
+	gtk_widget_set_name(GTK_WIDGET(entry), valid ? "" : "entryInvalid");
 #else
 	if (valid) {
 		GdkColor white = { 0, 0xFFFF, 0xFFFF, 0xFFFF};
@@ -132,6 +126,9 @@ void WComboBoxEntry::RemoveText(int position) {
 void WComboBoxEntry::AppendText(const char *text) {
 #if GTK_CHECK_VERSION(3,0,0)
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(GetID()), text);
+#if GTK_CHECK_VERSION(3,14,0)
+	gtk_combo_box_set_button_sensitivity(GTK_COMBO_BOX(GetID()), GTK_SENSITIVITY_ON);
+#endif
 #else
 	gtk_combo_box_append_text(GTK_COMBO_BOX(GetID()), text);
 #endif
@@ -140,6 +137,9 @@ void WComboBoxEntry::AppendText(const char *text) {
 void WComboBoxEntry::ClearList() {
 #if GTK_CHECK_VERSION(3,0,0)
 	gtk_combo_box_text_remove_all(GTK_COMBO_BOX_TEXT(GetID()));
+#if GTK_CHECK_VERSION(3,14,0)
+	gtk_combo_box_set_button_sensitivity(GTK_COMBO_BOX(GetID()), GTK_SENSITIVITY_OFF);
+#endif
 #else
 	for (int i = 0; i < 10; i++) {
 		RemoveText(0);
@@ -513,7 +513,11 @@ void WTable::Add(GtkWidget *child, int width, bool expand, int xpadding, int ypa
 	if (child) {
 #if USE_GRID
 		gtk_widget_set_hexpand(child, expand);
+#if GTK_CHECK_VERSION(3,14,0)
+		gtk_widget_set_margin_end(child, xpadding);
+#else
 		gtk_widget_set_margin_right(child, xpadding);
+#endif
 		gtk_widget_set_margin_bottom(child, ypadding);
 		gtk_grid_attach(GTK_GRID(GetID()), child,
 			next % columns, next / columns,
@@ -535,7 +539,12 @@ void WTable::Add(GtkWidget *child, int width, bool expand, int xpadding, int ypa
 }
 
 void WTable::Label(GtkWidget *child) {
+#if GTK_CHECK_VERSION(3,14,0)
+	gtk_widget_set_halign(child, GTK_ALIGN_END);
+	gtk_widget_set_valign(child, GTK_ALIGN_BASELINE);
+#else
 	gtk_misc_set_alignment(GTK_MISC(child), 1.0, 0.5);
+#endif
 	Add(child);
 }
 
