@@ -33,6 +33,7 @@
 #include <QTextLayout>
 #include <QTextLine>
 #include <QLibrary>
+#include <QElapsedTimer>
 #include <cstdio>
 
 #ifdef SCI_NAMESPACE
@@ -181,7 +182,7 @@ void SurfaceImpl::Init(SurfaceID sid, WindowID /*wid*/)
 
 void SurfaceImpl::InitPixMap(int width,
         int height,
-        Surface * /*surface*/,
+        Surface *surface,
         WindowID /*wid*/)
 {
 	Release();
@@ -189,6 +190,9 @@ void SurfaceImpl::InitPixMap(int width,
 	if (height < 1) height = 1;
 	deviceOwned = true;
 	device = new QPixmap(width, height);
+	SurfaceImpl *psurfOther = static_cast<SurfaceImpl *>(surface);
+	SetUnicodeMode(psurfOther->unicodeMode);
+	SetDBCSMode(psurfOther->codePage);
 }
 
 void SurfaceImpl::Release()
@@ -292,15 +296,13 @@ void SurfaceImpl::RectangleDraw(PRectangle rc,
 {
 	PenColour(fore);
 	BrushColour(back);
-	QRect rect = QRect(rc.left, rc.top, rc.Width() - 1, rc.Height() - 1);
+	QRectF rect(rc.left, rc.top, rc.Width() - 1, rc.Height() - 1);
 	GetPainter()->drawRect(rect);
 }
 
 void SurfaceImpl::FillRectangle(PRectangle rc, ColourDesired back)
 {
-	BrushColour(back);
-	GetPainter()->setPen(Qt::NoPen);
-	GetPainter()->drawRect(QRectFromPRect(rc));
+	GetPainter()->fillRect(QRectFFromPRect(rc), QColorFromCA(back));
 }
 
 void SurfaceImpl::FillRectangle(PRectangle rc, Surface &surfacePattern)
@@ -328,7 +330,7 @@ void SurfaceImpl::RoundedRectangle(PRectangle rc,
 {
 	PenColour(fore);
 	BrushColour(back);
-	GetPainter()->drawRoundRect(QRectFromPRect(rc));
+	GetPainter()->drawRoundRect(QRectFFromPRect(rc));
 }
 
 void SurfaceImpl::AlphaRectangle(PRectangle rc,
@@ -349,7 +351,7 @@ void SurfaceImpl::AlphaRectangle(PRectangle rc,
 
 	// A radius of 1 shows no curve so add 1
 	qreal radius = cornerSize+1;
-	QRect rect(rc.left, rc.top, rc.Width() - 1, rc.Height() - 1);
+	QRectF rect(rc.left, rc.top, rc.Width() - 1, rc.Height() - 1);
 	GetPainter()->drawRoundedRect(rect, radius, radius);
 }
 
@@ -377,7 +379,7 @@ void SurfaceImpl::Ellipse(PRectangle rc,
 {
 	PenColour(fore);
 	BrushColour(back);
-	GetPainter()->drawEllipse(QRectFromPRect(rc));
+	GetPainter()->drawEllipse(QRectFFromPRect(rc));
 }
 
 void SurfaceImpl::Copy(PRectangle rc, Point from, Surface &surfaceSource)
@@ -435,7 +437,7 @@ void SurfaceImpl::DrawTextTransparent(PRectangle rc,
 
 void SurfaceImpl::SetClip(PRectangle rc)
 {
-	GetPainter()->setClipRect(QRectFromPRect(rc));
+	GetPainter()->setClipRect(QRectFFromPRect(rc));
 }
 
 static size_t utf8LengthFromLead(unsigned char uch)
@@ -474,11 +476,11 @@ void SurfaceImpl::MeasureWidths(Font &font,
 			int codeUnits = (lenChar < 4) ? 1 : 2;
 			qreal xPosition = tl.cursorToX(ui+codeUnits);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
-				positions[i++] = qRound(xPosition);
+				positions[i++] = xPosition;
 			}
 			ui += codeUnits;
 		}
-		int lastPos = 0;
+		XYPOSITION lastPos = 0;
 		if (i > 0)
 			lastPos = positions[i-1];
 		while (i<len) {
@@ -491,21 +493,21 @@ void SurfaceImpl::MeasureWidths(Font &font,
 			size_t lenChar = Platform::IsDBCSLeadByte(codePage, s[i]) ? 2 : 1;
 			qreal xPosition = tl.cursorToX(ui+1);
 			for (unsigned int bytePos=0; (bytePos<lenChar) && (i<len); bytePos++) {
-				positions[i++] = qRound(xPosition);
+				positions[i++] = xPosition;
 			}
 			ui++;
 		}
 	} else {
 		// Single byte encoding
 		for (int i=0; i<len; i++) {
-			positions[i] = qRound(tl.cursorToX(i+1));
+			positions[i] = tl.cursorToX(i+1);
 		}
 	}
 }
 
 XYPOSITION SurfaceImpl::WidthText(Font &font, const char *s, int len)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	SetCodec(font);
 	QString string = codec->toUnicode(s, len);
 	return metrics.width(string);
@@ -513,19 +515,19 @@ XYPOSITION SurfaceImpl::WidthText(Font &font, const char *s, int len)
 
 XYPOSITION SurfaceImpl::WidthChar(Font &font, char ch)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	return metrics.width(ch);
 }
 
 XYPOSITION SurfaceImpl::Ascent(Font &font)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	return metrics.ascent();
 }
 
 XYPOSITION SurfaceImpl::Descent(Font &font)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	// Qt returns 1 less than true descent
 	// See: QFontEngineWin::descent which says:
 	// ### we subtract 1 to even out the historical +1 in QFontMetrics's
@@ -540,19 +542,19 @@ XYPOSITION SurfaceImpl::InternalLeading(Font & /* font */)
 
 XYPOSITION SurfaceImpl::ExternalLeading(Font &font)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	return metrics.leading();
 }
 
 XYPOSITION SurfaceImpl::Height(Font &font)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	return metrics.height();
 }
 
 XYPOSITION SurfaceImpl::AverageCharWidth(Font &font)
 {
-	QFontMetrics metrics(*FontPointer(font), device);
+	QFontMetricsF metrics(*FontPointer(font), device);
 	return metrics.averageCharWidth();
 }
 
@@ -781,7 +783,7 @@ private:
 
 class ListWidget : public QListWidget {
 public:
-	ListWidget(QWidget *parent);
+	explicit ListWidget(QWidget *parent);
 	virtual ~ListWidget();
 
 	void setDoubleClickAction(CallBackAction action, void *data);
@@ -1111,7 +1113,7 @@ class DynamicLibraryImpl : public DynamicLibrary {
 protected:
 	QLibrary *lib;
 public:
-	DynamicLibraryImpl(const char *modulePath) {
+	explicit DynamicLibraryImpl(const char *modulePath) {
 		QString path = QString::fromUtf8(modulePath);
 		lib = new QLibrary(path);
 	}
@@ -1304,21 +1306,28 @@ int Platform::DBCSCharMaxLength()
 
 //----------------------------------------------------------------------
 
-static QTime timer;
+static QElapsedTimer timer;
 
 ElapsedTime::ElapsedTime() : bigBit(0), littleBit(0)
 {
-	timer.start();
+	if (!timer.isValid()) {
+		timer.start();
+	}
+	qint64 ns64Now = timer.nsecsElapsed();
+	bigBit = static_cast<unsigned long>(ns64Now >> 32);
+	littleBit = static_cast<unsigned long>(ns64Now & 0xFFFFFFFF);
 }
 
 double ElapsedTime::Duration(bool reset)
 {
-	double result = timer.elapsed();
+	qint64 ns64Now = timer.nsecsElapsed();
+	qint64 ns64Start = (static_cast<qint64>(static_cast<unsigned long>(bigBit)) << 32) + static_cast<unsigned long>(littleBit);
+	double result = ns64Now - ns64Start;
 	if (reset) {
-		timer.restart();
+		bigBit = static_cast<unsigned long>(ns64Now >> 32);
+		littleBit = static_cast<unsigned long>(ns64Now & 0xFFFFFFFF);
 	}
-	result /= 1000.0;
-	return result;
+	return result / 1000000000.0;	// 1 billion nanoseconds in a second
 }
 
 #ifdef SCI_NAMESPACE
