@@ -29,7 +29,7 @@
 #include <io.h>
 
 #undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
+#define _WIN32_WINNT  0x0602
 #include <windows.h>
 #include <commctrl.h>
 
@@ -42,7 +42,6 @@
 
 #include "GUI.h"
 
-#include "SString.h"
 #include "FilePath.h"
 
 #if defined(__unix__)
@@ -197,7 +196,7 @@ FilePath FilePath::BaseName() const {
 FilePath FilePath::Extension() const {
 	size_t dirEnd = fileName.rfind(pathSepChar);
 	size_t extStart = fileName.rfind('.');
-	if ((extStart != GUI::gui_string::npos) && (extStart > dirEnd))
+	if ((extStart != GUI::gui_string::npos) && ((dirEnd == GUI::gui_string::npos) || (extStart > dirEnd)))
 		return fileName.substr(extStart + 1);
 	else
 		return GUI_TEXT("");
@@ -417,6 +416,24 @@ FILE *FilePath::Open(const GUI::gui_char *mode) const {
 	}
 }
 
+/// Size of block for file reading.
+static const size_t readBlockSize = 64 * 1024;
+
+std::vector<char> FilePath::Read() const {
+	std::vector<char> data;
+	FILE *fp = Open(fileRead);
+	if (fp) {
+		std::vector<char> block(readBlockSize);
+		size_t lenBlock = fread(&block[0], 1, block.size(), fp);
+		while (lenBlock > 0) {
+			data.insert(data.end(), block.begin(), block.begin() + lenBlock);
+			lenBlock = fread(&block[0], 1, block.size(), fp);
+		}
+		fclose(fp);
+	}
+	return data;
+}
+
 void FilePath::Remove() const {
 	unlink(AsInternal());
 }
@@ -594,16 +611,7 @@ void FilePath::FixName() {
 std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *directoryForRun) {
 	std::string output;
 #ifdef _WIN32
-	SECURITY_ATTRIBUTES sa = {sizeof(SECURITY_ATTRIBUTES), 0, 0};
-	sa.bInheritHandle = TRUE;
-	sa.lpSecurityDescriptor = NULL;
-
-	SECURITY_DESCRIPTOR sd;
-	// Make a real security thing to allow inheriting handles
-	::InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
-	::SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
-	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
-	sa.lpSecurityDescriptor = &sd;
+	SECURITY_ATTRIBUTES sa = {sizeof(SECURITY_ATTRIBUTES), NULL, TRUE};
 
 	HANDLE hPipeWrite = NULL;
 	HANDLE hPipeRead = NULL;
@@ -653,7 +661,7 @@ std::string CommandExecute(const GUI::gui_char *command, const GUI::gui_char *di
 
 		DWORD bytesRead = 0;
 		DWORD bytesAvail = 0;
-		char buffer[16 * 1024];
+		char buffer[8 * 1024];
 
 		if (::PeekNamedPipe(hPipeRead, buffer, sizeof(buffer), &bytesRead, &bytesAvail, NULL)) {
 			if (bytesAvail > 0) {
