@@ -32,23 +32,23 @@ struct JobMode {
 	int saveBefore;
 	bool isFilter;
 	int flags;
-	SString input;
+	std::string input;
 	JobMode(PropSetFile &props, int item, const char *fileNameExt);
 };
 
 class Job {
 public:
-	SString command;
+	std::string command;
 	FilePath directory;
 	JobSubsystem jobType;
-	SString input;
+	std::string input;
 	int flags;
 
 	Job() {
 		Clear();
 	}
 
-	Job(const SString &command_, const FilePath &directory_, JobSubsystem jobType_, const SString &input_, int flags_)
+	Job(const std::string &command_, const FilePath &directory_, JobSubsystem jobType_, const std::string &input_, int flags_)
 		: command(command_), directory(directory_), jobType(jobType_), input(input_), flags(flags_) {
 	}
 
@@ -134,7 +134,7 @@ public:
 	}
 
 	void ClearJobs();
-	void AddCommand(const SString &command, const FilePath &directory, JobSubsystem jobType, const SString &input, int flags);
+	void AddCommand(const std::string &command, const FilePath &directory, JobSubsystem jobType, const std::string &input, int flags);
 };
 
 #endif
