@@ -15,7 +15,7 @@
 #include "Scintilla.h"
 
 #include "GUI.h"
-#include "SString.h"
+#include "StringHelpers.h"
 #include "FilePath.h"
 #include "StyleWriter.h"
 #include "Extender.h"
@@ -254,9 +254,9 @@ static int cf_scite_constname(lua_State *L) {
 static int cf_scite_open(lua_State *L) {
 	const char *s = luaL_checkstring(L, 1);
 	if (s) {
-		SString cmd = "open:";
+		std::string cmd = "open:";
 		cmd += s;
-		cmd.substitute("\\", "\\\\");
+		Substitute(cmd, "\\", "\\\\");
 		host->Perform(cmd.c_str());
 	}
 	return 0;
@@ -786,6 +786,16 @@ static bool call_function(lua_State *L, int nargs, bool ignoreFunctionReturnValu
 	return handled;
 }
 
+static bool HasNamedFunction(const char *name) {
+	bool hasFunction = false;
+	if (luaState) {
+		lua_getglobal(luaState, name);
+		hasFunction = lua_isfunction(luaState, -1);
+		lua_pop(luaState, 1);
+	}
+	return hasFunction;
+}
+
 static bool CallNamedFunction(const char *name) {
 	bool handled = false;
 	if (luaState) {
@@ -1404,7 +1414,7 @@ static bool InitGlobalScope(bool checkProperties, bool forceReload = false) {
 		// that should be blocked during startup, e.g. the ones that allow
 		// you to add or switch buffers?
 
-		FilePath fpTest(GUI::StringFromUTF8(startupScript.c_str()));
+		FilePath fpTest(GUI::StringFromUTF8(startupScript));
 		if (fpTest.Exists()) {
 			luaL_loadfile(luaState, startupScript.c_str());
 			if (!call_function(luaState, 0, true)) {
@@ -1629,7 +1639,7 @@ bool LuaExtension::OnSave(const char *filename) {
 	bool result = CallNamedFunction("OnSave", filename);
 
 	FilePath fpSaving = FilePath(GUI::StringFromUTF8(filename)).NormalizePath();
-	if (startupScript.length() && fpSaving == FilePath(GUI::StringFromUTF8(startupScript.c_str())).NormalizePath()) {
+	if (startupScript.length() && fpSaving == FilePath(GUI::StringFromUTF8(startupScript)).NormalizePath()) {
 		if (GetPropertyInt("ext.lua.auto.reload") > 0) {
 			InitGlobalScope(false, true);
 			if (extensionScript.length()) {
@@ -2060,3 +2070,7 @@ bool LuaExtension::OnClose(const char *filename) {
 bool LuaExtension::OnUserStrip(int control, int change) {
 	return CallNamedFunction("OnStrip", control, change);
 }
+
+bool LuaExtension::NeedsOnClose() {
+	return HasNamedFunction("OnClose");
+}
