@@ -9,10 +9,10 @@
 #define SCITE_H
 
 // Version numbers and dates
-#define VERSION_SCITE "3.5.0"
-#define VERSION_WORDS 3, 5, 0, 0
-#define COPYRIGHT_DATES "December 1998-August 2014"
-#define COPYRIGHT_YEARS "1998-2014"
+#define VERSION_SCITE "3.6.0"
+#define VERSION_WORDS 3, 6, 0, 0
+#define COPYRIGHT_DATES "December 1998-August 2015"
+#define COPYRIGHT_YEARS "1998-2015"
 
 // Menu IDs.
 // These are located 100 apart. No one will want more than 100 in each menu ;)
@@ -102,6 +102,8 @@
 #define IDM_DUPLICATE	250
 #define IDM_INCSEARCH 		252
 #define IDM_ENTERSELECTION  256
+#define IDM_SELECTIONADDNEXT  257
+#define IDM_SELECTIONADDEACH  258
 
 #define IDC_INCFINDTEXT     253
 #define IDC_INCFINDBTNOK	254
