@@ -171,7 +171,7 @@ public:
 	void RemoveInvisible(int index);
 	void RemoveCurrent();
 	int Current() const;
-	Buffer *CurrentBuffer();
+	Buffer *CurrentBuffer() const;
 	void SetCurrent(int index);
 	int StackNext();
 	int StackPrev();
@@ -232,7 +232,7 @@ enum IndentationStatus {
 
 struct StyleAndWords {
 	int styleNumber;
-	SString words;
+	std::string words;
 	StyleAndWords() : styleNumber(0) {
 	}
 	bool IsEmpty() const { return words.length() == 0; }
@@ -260,13 +260,13 @@ struct CurrentWordHighlight {
 };
 
 class Localization : public PropSetFile, public ILocalize {
-	SString missing;
+	std::string missing;
 public:
 	bool read;
 	Localization() : PropSetFile(true), read(false) {
 	}
 	GUI::gui_string Text(const char *s, bool retainIfNotFound=true);
-	void SetMissing(const SString &missing_) {
+	void SetMissing(const std::string &missing_) {
 		missing = missing_;
 	}
 };
@@ -361,8 +361,8 @@ protected:
 	enum { markerBookmark = 1 };
 	ComboMemory memFiles;
 	ComboMemory memDirectory;
-	SString parameterisedCommand;
-	SString abbrevInsert;
+	std::string parameterisedCommand;
+	std::string abbrevInsert;
 
 	enum { languageCmdID = IDM_LANGUAGE };
 	std::vector<LanguageMenuItem> languageMenu;
@@ -372,11 +372,12 @@ protected:
 
 	int codePage;
 	int characterSet;
-	SString language;
+	std::string language;
 	int lexLanguage;
+	std::vector<char> subStyleBases;
 	int lexLPeg;
 	StringList apis;
-	SString apisFileNames;
+	std::string apisFileNames;
 	std::string functionDefinition;
 
 	int diagnosticStyleStart;
@@ -409,7 +410,7 @@ protected:
 	bool tabHideOne; // Hide tab bar if one buffer is opened only
 	bool tabMultiLine;
 	bool sbVisible;	///< @c true if status bar is visible.
-	SString sbValue;	///< Status bar text.
+	std::string sbValue;	///< Status bar text.
 	int sbNum;	///< Number of the currently displayed status bar information.
 	int visHeightTools;
 	int visHeightTab;
@@ -452,22 +453,23 @@ protected:
 	bool indentationWSVisible;
 	int indentExamine;
 	bool autoCompleteIgnoreCase;
+	bool imeAutoComplete;
 	bool callTipUseEscapes;
 	bool callTipIgnoreCase;
 	bool autoCCausedByOnlyOne;
-	SString calltipWordCharacters;
-	SString calltipParametersStart;
-	SString calltipParametersEnd;
-	SString calltipParametersSeparators;
-	SString calltipEndDefinition;
-	SString autoCompleteStartCharacters;
-	SString autoCompleteFillUpCharacters;
-	SString wordCharacters;
-	SString whitespaceCharacters;
+	std::string calltipWordCharacters;
+	std::string calltipParametersStart;
+	std::string calltipParametersEnd;
+	std::string calltipParametersSeparators;
+	std::string calltipEndDefinition;
+	std::string autoCompleteStartCharacters;
+	std::string autoCompleteFillUpCharacters;
+	std::string wordCharacters;
+	std::string whitespaceCharacters;
 	int startCalltipWord;
 	int currentCallTip;
 	int maxCallTips;
-	SString currentCallTipWord;
+	std::string currentCallTipWord;
 	int lastPosCallTip;
 
 	bool margin;
@@ -489,7 +491,7 @@ protected:
 	JobQueue jobQueue;
 
 	bool macrosEnabled;
-	SString currentMacro;
+	std::string currentMacro;
 	bool recording;
 
 	PropSetFile propsEmbed;
@@ -521,7 +523,7 @@ protected:
 	bool IsBufferAvailable() const;
 	bool CanMakeRoom(bool maySaveIfDirty = true);
 	void SetDocumentAt(int index, bool updateStack = true);
-	Buffer *CurrentBuffer() {
+	Buffer *CurrentBuffer() const {
 		return buffers.CurrentBuffer();
 	}
 	void SetBuffersMenu();
@@ -548,11 +550,11 @@ protected:
 	int CallFocusedElseDefault(int defaultValue, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	sptr_t CallPane(int destination, unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
 	void CallChildren(unsigned int msg, uptr_t wParam = 0, sptr_t lParam = 0);
-	SString GetTranslationToAbout(const char * const propname, bool retainIfNotFound = true);
+	std::string GetTranslationToAbout(const char * const propname, bool retainIfNotFound = true);
 	int LengthDocument();
 	int GetCaretInLine();
 	void GetLine(char *text, int sizeText, int line = -1);
-	SString GetLine(int line = -1);
+	std::string GetCurrentLine();
 	static void GetRange(GUI::ScintillaWindow &win, int start, int end, char *text);
 	int IsLinePreprocessorCondition(char *line);
 	bool FindMatchingPreprocessorCondition(int &curLine, int direction, int condEnd1, int condEnd2);
@@ -586,7 +588,7 @@ protected:
 	void DiscoverEOLSetting();
 	void DiscoverIndentSetting();
 	std::string DiscoverLanguage();
-	void OpenFile(long fileSize, bool suppressMessage, bool asynchronous);
+	void OpenCurrentFile(long fileSize, bool suppressMessage, bool asynchronous);
 	virtual void OpenUriList(const char *) {}
 	virtual bool OpenDialog(FilePath directory, const GUI::gui_char *filesFilter) = 0;
 	virtual bool SaveAsDialog() = 0;
@@ -653,7 +655,7 @@ protected:
 	FilePath GetLocalPropertiesFileName();
 	FilePath GetAbbrevPropertiesFileName();
 	void OpenProperties(int propsFile);
-	static int GetMenuCommandAsInt(SString commandName);
+	static int GetMenuCommandAsInt(std::string commandName);
 	virtual void Print(bool) {}
 	virtual void PrintSetup() {}
 	virtual void UserStripShow(const char * /* description */) {}
@@ -664,19 +666,21 @@ protected:
 	Sci_CharacterRange GetSelection();
 	SelectedRange GetSelectedRange();
 	void SetSelection(int anchor, int currentPos);
-	SString GetCTag();
-	static SString GetRange(GUI::ScintillaWindow &win, int selStart, int selEnd);
-	virtual SString GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
-	static SString GetLine(GUI::ScintillaWindow &win, int line);
+	std::string GetCTag();
+	static std::string GetRangeString(GUI::ScintillaWindow &win, int selStart, int selEnd);
+	virtual std::string GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd);
+	static std::string GetLine(GUI::ScintillaWindow &win, int line);
 	void RangeExtend(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
 		bool (SciTEBase::*ischarforsel)(char ch));
-	SString RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
+	std::string RangeExtendAndGrab(GUI::ScintillaWindow &wCurrent, int &selStart, int &selEnd,
 		bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
-	SString SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
-	SString SelectionWord(bool stripEol = true);
-	SString SelectionFilename();
+	std::string SelectionExtend(bool (SciTEBase::*ischarforsel)(char ch), bool stripEol = true);
+	std::string SelectionWord(bool stripEol = true);
+	std::string SelectionFilename();
 	void SelectionIntoProperties();
 	void SelectionIntoFind(bool stripEol = true);
+	enum AddSelection { addNext, addEach };
+	void SelectionAdd(AddSelection add);
 	virtual std::string EncodeString(const std::string &s);
 	virtual void Find() = 0;
 	enum MessageBoxChoice {
@@ -695,7 +699,8 @@ protected:
 		mbsIconWarning = 0x30
 	};
 	virtual MessageBoxChoice WindowMessageBox(GUI::Window &w, const GUI::gui_string &msg, MessageBoxStyle style = mbsIconWarning) = 0;
-	virtual void FindMessageBox(const SString &msg, const std::string *findItem = 0) = 0;
+	virtual void FindMessageBox(const std::string &msg, const std::string *findItem = 0) = 0;
+	bool FindReplaceAdvanced() const;
 	int FindInTarget(std::string findWhatText, int startPosition, int endPosition);
 	virtual void SetFindText(const char *sFind);
 	virtual void SetFind(const char *sFind);
@@ -724,7 +729,7 @@ protected:
 	virtual void ParamGrab() = 0;
 	virtual bool ParametersDialog(bool modal) = 0;
 	bool HandleXml(char ch);
-	static SString FindOpenXmlTag(const char sel[], int nSize);
+	static std::string FindOpenXmlTag(const char sel[], int nSize);
 	void GoMatchingBrace(bool select);
 	void GoMatchingPreprocCond(int direction, bool select);
 	virtual void FindReplace(bool replace) = 0;
@@ -736,11 +741,11 @@ protected:
 	void ShowMessages(int line);
 	void GoMessage(int dir);
 	virtual bool StartCallTip();
-	char *GetNearestWords(const char *wordStart, size_t searchLen,
+	std::string GetNearestWords(const char *wordStart, size_t searchLen,
 		const char *separators, bool ignoreCase=false, bool exactLen=false);
 	virtual void FillFunctionDefinition(int pos = -1);
 	void ContinueCallTip();
-	virtual void EliminateDuplicateWords(char *words);
+	virtual void EliminateDuplicateWords(std::string &words);
 	virtual bool StartAutoComplete();
 	virtual bool StartAutoCompleteWord(bool onlyOneWord);
 	virtual bool StartExpandAbbreviation();
@@ -749,7 +754,7 @@ protected:
 	virtual bool StartBlockComment();
 	virtual bool StartBoxComment();
 	virtual bool StartStreamComment();
-	unsigned int GetLinePartsInStyle(int line, int style1, int style2, SString sv[], int len);
+	unsigned int GetLinePartsInStyle(int line, int style1, int style2, std::string sv[], int len);
 	void SetLineIndentation(int line, int indent);
 	int GetLineIndentation(int line);
 	int GetLineIndentPosition(int line);
@@ -759,7 +764,7 @@ protected:
 	int IndentOfBlock(int line);
 	void MaintainIndentation(char ch);
 	void AutomaticIndentation(char ch);
-	void CharAdded(char ch);
+	void CharAdded(int ch);
 	void CharAddedOutput(int ch);
 	void SetTextProperties(PropSetFile &ps);
 	virtual void SetFileProperties(PropSetFile &ps) = 0;
@@ -767,8 +772,8 @@ protected:
 	int GetLineLength(int line);
 	int GetCurrentLineNumber();
 	int GetCurrentScrollPosition();
-	virtual void AddCommand(const SString &cmd, const SString &dir,
-	        JobSubsystem jobType, const SString &input = "",
+	virtual void AddCommand(const std::string &cmd, const std::string &dir,
+	        JobSubsystem jobType, const std::string &input = "",
 	        int flags = 0);
 	virtual void AboutDialog() = 0;
 	virtual void QuitProgram() = 0;
@@ -792,7 +797,7 @@ protected:
 	bool MarginClick(int position, int modifiers);
 	void NewLineInOutput();
 	virtual void SetStatusBarText(const char *s) = 0;
-	virtual void Notify(SCNotification *notification);
+	virtual void Notify(const SCNotification *notification);
 	virtual void ShowToolBar() = 0;
 	virtual void ShowTabBar() = 0;
 	virtual void ShowStatusBar() = 0;
@@ -836,7 +841,7 @@ protected:
 	        const char *text, const char *mnemonic);
 	bool ToolIsImmediate(int item);
 	void SetToolsMenu();
-	JobSubsystem SubsystemType(const char *cmd, int item = -1);
+	JobSubsystem SubsystemType(const char *cmd);
 	void ToolsMenu(int item);
 
 	void AssignKey(int key, int mods, int cmd);
@@ -849,18 +854,20 @@ protected:
 	GUI::gui_string LocaliseMessage(const char *s,
 		const GUI::gui_char *param0 = 0, const GUI::gui_char *param1 = 0, const GUI::gui_char *param2 = 0);
 	virtual void ReadLocalization();
-	SString GetFileNameProperty(const char *name);
+	std::string GetFileNameProperty(const char *name);
 	virtual void ReadPropertiesInitial();
 	void ReadFontProperties();
 	void SetOverrideLanguage(int cmdID);
 	StyleAndWords GetStyleAndWords(const char *base);
-	SString ExtensionFileName();
+	std::string ExtensionFileName() const;
 	static const char *GetNextPropItem(const char *pStart, char *pPropItem, int maxLen);
 	void ForwardPropertyToEditor(const char *key);
 	void DefineMarker(int marker, int markerType, Colour fore, Colour back, Colour backSelected);
-	void ReadAPI(const SString &fileNameForExtension);
-	SString FindLanguageProperty(const char *pattern, const char *defaultValue = "");
+	void ReadAPI(const std::string &fileNameForExtension);
+	std::string FindLanguageProperty(const char *pattern, const char *defaultValue = "");
 	virtual void ReadProperties();
+	std::string StyleString(const char *lang, int style) const;
+	StyleDefinition StyleDefinitionFor(int style);
 	void SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd);
 	void SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int start, int last);
 	void SetStyleFor(GUI::ScintillaWindow &win, const char *lang);
@@ -886,7 +893,7 @@ protected:
 	void StartRecordMacro();
 	void StopRecordMacro();
 	void StartPlayMacro();
-	bool RecordMacroCommand(SCNotification *notification);
+	bool RecordMacroCommand(const SCNotification *notification);
 	void ExecuteMacroCommand(const char *command);
 	void AskMacroList();
 	bool StartMacroList(const char *words);
@@ -935,7 +942,7 @@ public:
 
 	enum { maxParam = 4 };
 
-	SciTEBase(Extension *ext = 0);
+	explicit SciTEBase(Extension *ext = 0);
 	virtual ~SciTEBase();
 
 	void ProcessExecute();
@@ -947,7 +954,7 @@ public:
 
 private:
 	// un-implemented copy-constructor and assignment operator
-	SciTEBase(const SciTEBase&);
+	explicit SciTEBase(const SciTEBase&);
 	void operator=(const SciTEBase&);
 };
 
