@@ -17,6 +17,7 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <algorithm>
 
 #include "Scintilla.h"
 #include "SciLexer.h"
@@ -24,7 +25,6 @@
 
 #include "GUI.h"
 
-#include "SString.h"
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
@@ -182,7 +182,7 @@ int BufferList::Current() const {
 	return current;
 }
 
-Buffer *BufferList::CurrentBuffer() {
+Buffer *BufferList::CurrentBuffer() const {
 	return &buffers[Current()];
 }
 
@@ -533,10 +533,10 @@ FilePath SciTEBase::UserFilePath(const GUI::gui_char *name) {
 	return FilePath(GetSciteUserHome(), nameWithVisibility.c_str());
 }
 
-static SString IndexPropKey(const char *bufPrefix, int bufIndex, const char *bufAppendix) {
-	SString pKey = bufPrefix;
+static std::string IndexPropKey(const char *bufPrefix, int bufIndex, const char *bufAppendix) {
+	std::string pKey = bufPrefix;
 	pKey += '.';
-	pKey += SString(bufIndex + 1);
+	pKey += StdStringFromInteger(bufIndex + 1);
 	if (bufAppendix != NULL) {
 		pKey += ".";
 		pKey += bufAppendix;
@@ -566,26 +566,26 @@ void SciTEBase::RestoreRecentMenu() {
 	DeleteFileStackMenu();
 
 	for (int i = 0; i < fileStackMax; i++) {
-		SString propKey = IndexPropKey("mru", i, "path");
-		SString propStr = propsSession.Get(propKey.c_str());
+		std::string propKey = IndexPropKey("mru", i, "path");
+		std::string propStr = propsSession.GetString(propKey.c_str());
 		if (propStr == "")
 			continue;
-		AddFileToStack(GUI::StringFromUTF8(propStr.c_str()), sr, 0);
+		AddFileToStack(GUI::StringFromUTF8(propStr), sr, 0);
 	}
 }
 
-static std::vector<int> LinesFromString(const SString &s) {
+static std::vector<int> LinesFromString(const std::string &s) {
 	std::vector<int> result;
 	if (s.length()) {
-		char *buf = new char[s.length() + 1];
-		strcpy(buf, s.c_str());
-		char *p = strtok(buf, ",");
-		while (p != NULL) {
-			int line = atoi(p) - 1;
+		size_t start = 0;
+		for (;;) {
+			const int line = atoi(s.c_str() + start) - 1;
 			result.push_back(line);
-			p = strtok(NULL, ",");
+			const size_t posComma = s.find(',', start);
+			if (posComma == std::string::npos)
+				break;
+			start = posComma + 1;
 		}
-		delete []buf;
 	}
 	return result;
 }
@@ -601,16 +601,16 @@ void SciTEBase::RestoreFromSession(const Session &session) {
 void SciTEBase::RestoreSession() {
 	if (props.GetInt("save.find") != 0) {
 		for (int i = 0;; i++) {
-			SString propKey = IndexPropKey("search", i, "findwhat");
-			SString propStr = propsSession.Get(propKey.c_str());
+			std::string propKey = IndexPropKey("search", i, "findwhat");
+			std::string propStr = propsSession.GetString(propKey.c_str());
 			if (propStr == "")
 				break;
 			memFinds.AppendList(propStr.c_str());
 		}
 
 		for (int i = 0;; i++) {
-			SString propKey = IndexPropKey("search", i, "replacewith");
-			SString propStr = propsSession.Get(propKey.c_str());
+			std::string propKey = IndexPropKey("search", i, "replacewith");
+			std::string propStr = propsSession.GetString(propKey.c_str());
 			if (propStr == "")
 				break;
 			memReplaces.AppendList(propStr.c_str());
@@ -623,18 +623,22 @@ void SciTEBase::RestoreSession() {
 	Session session;
 
 	for (int i = 0; i < bufferMax; i++) {
-		SString propKey = IndexPropKey("buffer", i, "path");
-		SString propStr = propsSession.Get(propKey.c_str());
+		std::string propKey = IndexPropKey("buffer", i, "path");
+		std::string propStr = propsSession.GetString(propKey.c_str());
 		if (propStr == "")
 			continue;
 
 		BufferState bufferState;
-		bufferState.Set(GUI::StringFromUTF8(propStr.c_str()));
+		bufferState.Set(GUI::StringFromUTF8(propStr));
 
 		propKey = IndexPropKey("buffer", i, "current");
 		if (propsSession.GetInt(propKey.c_str()))
 			session.pathActive = bufferState;
 
+		propKey = IndexPropKey("buffer", i, "scroll");
+		int scroll = propsSession.GetInt(propKey.c_str());
+		bufferState.scrollPosition = scroll;
+
 		propKey = IndexPropKey("buffer", i, "position");
 		int pos = propsSession.GetInt(propKey.c_str());
 
@@ -643,14 +647,14 @@ void SciTEBase::RestoreSession() {
 
 		if (props.GetInt("session.bookmarks")) {
 			propKey = IndexPropKey("buffer", i, "bookmarks");
-			propStr = propsSession.Get(propKey.c_str());
+			propStr = propsSession.GetString(propKey.c_str());
 			bufferState.bookmarks = LinesFromString(propStr);
 		}
 
 		if (props.GetInt("fold") && !props.GetInt("fold.on.open") &&
 			props.GetInt("session.folds")) {
 			propKey = IndexPropKey("buffer", i, "folds");
-			propStr = propsSession.Get(propKey.c_str());
+			propStr = propsSession.GetString(propKey.c_str());
 			bufferState.foldState = LinesFromString(propStr);
 		}
 
@@ -690,7 +694,7 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 	}
 
 	if (defaultSession && props.GetInt("save.recent")) {
-		SString propKey;
+		std::string propKey;
 		int j = 0;
 
 		fprintf(sessionFile, "\n");
@@ -705,7 +709,7 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 	}
 
 	if (defaultSession && props.GetInt("save.find")) {
-		SString propKey;
+		std::string propKey;
 		std::vector<std::string>::iterator it;
 		std::vector<std::string> mem = memFinds.AsVector();
 		if (!mem.empty()) {
@@ -734,13 +738,17 @@ void SciTEBase::SaveSessionFile(const GUI::gui_char *sessionName) {
 		for (int i = 0; i < buffers.lengthVisible; i++) {
 			if (buffers.buffers[i].IsSet() && !buffers.buffers[i].IsUntitled()) {
 				Buffer &buff = buffers.buffers[i];
-				SString propKey = IndexPropKey("buffer", i, "path");
+				std::string propKey = IndexPropKey("buffer", i, "path");
 				fprintf(sessionFile, "\n%s=%s\n", propKey.c_str(), buff.AsUTF8().c_str());
 
 				int pos = buff.selection.position + 1;
 				propKey = IndexPropKey("buffer", i, "position");
 				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), pos);
 
+				int scroll = buff.scrollPosition;
+				propKey = IndexPropKey("buffer", i, "scroll");
+				fprintf(sessionFile, "%s=%d\n", propKey.c_str(), scroll);
+
 				if (i == curr) {
 					propKey = IndexPropKey("buffer", i, "current");
 					fprintf(sessionFile, "%s=1\n", propKey.c_str());
@@ -794,32 +802,32 @@ void SciTEBase::SetIndentSettings() {
 	int tabSize = props.GetInt("tabsize");
 	int indentSize = props.GetInt("indent.size");
 	// Either set the settings related to the extension or the default ones
-	SString fileNameForExtension = ExtensionFileName();
-	SString useTabsChars = props.GetNewExpand("use.tabs.",
+	std::string fileNameForExtension = ExtensionFileName();
+	std::string useTabsChars = props.GetNewExpandString("use.tabs.",
 	        fileNameForExtension.c_str());
 	if (useTabsChars.length() != 0) {
-		wEditor.Call(SCI_SETUSETABS, useTabsChars.value());
+		wEditor.Call(SCI_SETUSETABS, atoi(useTabsChars.c_str()));
 	} else {
 		wEditor.Call(SCI_SETUSETABS, useTabs);
 	}
-	SString tabSizeForExt = props.GetNewExpand("tab.size.",
+	std::string tabSizeForExt = props.GetNewExpandString("tab.size.",
 	        fileNameForExtension.c_str());
 	if (tabSizeForExt.length() != 0) {
-		wEditor.Call(SCI_SETTABWIDTH, tabSizeForExt.value());
+		wEditor.Call(SCI_SETTABWIDTH, atoi(tabSizeForExt.c_str()));
 	} else if (tabSize != 0) {
 		wEditor.Call(SCI_SETTABWIDTH, tabSize);
 	}
-	SString indentSizeForExt = props.GetNewExpand("indent.size.",
+	std::string indentSizeForExt = props.GetNewExpandString("indent.size.",
 	        fileNameForExtension.c_str());
 	if (indentSizeForExt.length() != 0) {
-		wEditor.Call(SCI_SETINDENT, indentSizeForExt.value());
+		wEditor.Call(SCI_SETINDENT, atoi(indentSizeForExt.c_str()));
 	} else {
 		wEditor.Call(SCI_SETINDENT, indentSize);
 	}
 }
 
 void SciTEBase::SetEol() {
-	SString eol_mode = props.Get("eol.mode");
+	std::string eol_mode = props.GetString("eol.mode");
 	if (eol_mode == "LF") {
 		wEditor.Call(SCI_SETEOLMODE, SC_EOL_LF);
 	} else if (eol_mode == "CR") {
@@ -899,9 +907,9 @@ void SciTEBase::RestoreState(const Buffer &buffer, bool restoreBookmarks) {
 }
 
 void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew) {
-	bool closingLast = false;
+	bool closingLast = true;
 	int index = buffers.Current();
-	if (index >= 0) {
+	if ((index >= 0) && buffers.initialised) {
 		buffers.buffers[index].CancelLoad();
 	}
 
@@ -972,7 +980,7 @@ void SciTEBase::Close(bool updateUI, bool loadingSession, bool makingRoomForNew)
 		}
 	}
 
-	if (updateUI) {
+	if (updateUI && buffers.initialised) {
 		BuffersMenu();
 		UpdateStatusBar(true);
 	}
@@ -1232,7 +1240,7 @@ bool SciTEBase::AddFileToBuffer(const BufferState &bufferState) {
 		if (opened) {
 			int iBuffer = buffers.GetDocumentByName(bufferState, false);
 			if (iBuffer >= 0) {
-				buffers.buffers[iBuffer].scrollPosition = 0;
+				buffers.buffers[iBuffer].scrollPosition = bufferState.scrollPosition;
 				buffers.buffers[iBuffer].selection = bufferState.selection;
 				buffers.buffers[iBuffer].foldState = bufferState.foldState;
 				buffers.buffers[iBuffer].bookmarks = bufferState.bookmarks;
@@ -1386,13 +1394,13 @@ void SciTEBase::SetMenuItemLocalised(int menuNumber, int position, int itemID,
 }
 
 bool SciTEBase::ToolIsImmediate(int item) {
-	SString itemSuffix = item;
+	std::string itemSuffix = StdStringFromInteger(item);
 	itemSuffix += '.';
 
-	SString propName = "command.";
+	std::string propName = "command.";
 	propName += itemSuffix;
 
-	SString command = props.GetWild(propName.c_str(), FileNameExt().AsUTF8().c_str());
+	std::string command = props.GetWild(propName.c_str(), FileNameExt().AsUTF8().c_str());
 	if (command.length()) {
 		JobMode jobMode(props, item, FileNameExt().AsUTF8().c_str());
 		return jobMode.jobType == jobImmediate;
@@ -1407,22 +1415,22 @@ void SciTEBase::SetToolsMenu() {
 	int menuPos = TOOLS_START;
 	for (int item = 0; item < toolMax; item++) {
 		int itemID = IDM_TOOLS + item;
-		SString prefix = "command.name.";
-		prefix += SString(item);
+		std::string prefix = "command.name.";
+		prefix += StdStringFromInteger(item);
 		prefix += ".";
-		SString commandName = props.GetNewExpand(prefix.c_str(), FileNameExt().AsUTF8().c_str());
+		std::string commandName = props.GetNewExpandString(prefix.c_str(), FileNameExt().AsUTF8().c_str());
 		if (commandName.length()) {
-			SString sMenuItem = commandName;
+			std::string sMenuItem = commandName;
 			prefix = "command.shortcut.";
-			prefix += SString(item);
+			prefix += StdStringFromInteger(item);
 			prefix += ".";
-			SString sMnemonic = props.GetNewExpand(prefix.c_str(), FileNameExt().AsUTF8().c_str());
+			std::string sMnemonic = props.GetNewExpandString(prefix.c_str(), FileNameExt().AsUTF8().c_str());
 			if (item < 10 && sMnemonic.length() == 0) {
 				sMnemonic += "Ctrl+";
-				sMnemonic += SString(item);
+				sMnemonic += StdStringFromInteger(item);
 			}
 			SetMenuItemLocalised(menuTools, menuPos, itemID, sMenuItem.c_str(),
-				sMnemonic[0] ? sMnemonic.c_str() : NULL);
+				sMnemonic.length() ? sMnemonic.c_str() : NULL);
 			menuPos++;
 		}
 	}
@@ -1446,14 +1454,9 @@ void SciTEBase::SetToolsMenu() {
 	}
 }
 
-JobSubsystem SciTEBase::SubsystemType(const char *cmd, int item) {
-	SString subsysprefix = cmd;
-	if (item >= 0) {
-		subsysprefix += SString(item);
-		subsysprefix += ".";
-	}
-	SString subsystem = props.GetNewExpand(subsysprefix.c_str(), FileNameExt().AsUTF8().c_str());
-	return SubsystemFromChar(subsystem[0]);
+JobSubsystem SciTEBase::SubsystemType(const char *cmd) {
+	std::string subsystem = props.GetNewExpandString(cmd, FileNameExt().AsUTF8().c_str());
+	return subsystem.empty() ? jobCLI : SubsystemFromChar(subsystem.at(0));
 }
 
 void SciTEBase::ToolsMenu(int item) {
@@ -1544,10 +1547,16 @@ int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &co
 			}
 			const char *endPath = strchr(start, '(');
 			if (endPath) {
-				ptrdiff_t length = endPath - start;
-				sourcePath.assign(start, length);
-				endPath++;
-				return atoi(endPath) - 1;
+				if (!isdigitchar(endPath[1])) {
+					// This handles the common case of include files in the C:\Program Files (x86)\ directory
+					endPath = strchr(endPath + 1, '(');
+				}
+				if (endPath) {
+					ptrdiff_t length = endPath - start;
+					sourcePath.assign(start, length);
+					endPath++;
+					return atoi(endPath) - 1;
+				}
 			}
 			break;
 		}
@@ -1824,9 +1833,10 @@ int DecodeMessage(const char *cdoc, std::string &sourcePath, int format, int &co
 }
 
 // Remove up to and including ch
-static void Chomp(SString &s, int ch) {
-	if (s.contains(static_cast<char>(ch)))
-			s.remove(0, s.search(":") + 1);
+static void Chomp(std::string &s, int ch) {
+	const size_t posCh = s.find(static_cast<char>(ch));
+	if (posCh != std::string::npos)
+		s.erase(0, posCh + 1);
 }
 
 void SciTEBase::ShowMessages(int line) {
@@ -1840,22 +1850,22 @@ void SciTEBase::ShowMessages(int line) {
 	while ((line < maxLine) && (acc.StyleAt(acc.LineStart(line)) != SCE_ERR_CMD)) {
 		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, line, 0);
 		int lineEnd = wOutput.Call(SCI_GETLINEENDPOSITION, line, 0);
-		SString message = GetRange(wOutput, startPosLine, lineEnd);
+		std::string message = GetRangeString(wOutput, startPosLine, lineEnd);
 		std::string source;
 		int column;
-		char style = acc.StyleAt(startPosLine);
+		int style = acc.StyleAt(startPosLine);
 		int sourceLine = DecodeMessage(message.c_str(), source, style, column);
 		Chomp(message, ':');
 		if (style == SCE_ERR_GCC) {
 			Chomp(message, ':');
 		}
-		GUI::gui_string sourceString = GUI::StringFromUTF8(source.c_str());
+		GUI::gui_string sourceString = GUI::StringFromUTF8(source);
 		FilePath sourcePath = FilePath(sourceString).NormalizePath();
 		if (filePath.Name().SameNameAs(sourcePath.Name())) {
 			if (style == SCE_ERR_GCC) {
 				const char *sColon = strchr(message.c_str(), ':');
 				if (sColon) {
-					SString editLine = GetLine(wEditor, sourceLine);
+					std::string editLine = GetLine(wEditor, sourceLine);
 					if (editLine == (sColon+1)) {
 						line++;
 						continue;
@@ -1875,11 +1885,11 @@ void SciTEBase::ShowMessages(int line) {
 				// Only append unique messages
 				msgCurrent += message.c_str();
 				int msgStyle = 0;
-				if (message.search("warning") >= 0)
+				if (message.find("warning") != std::string::npos)
 					msgStyle = 1;
-				if (message.search("error") >= 0)
+				if (message.find("error") != std::string::npos)
 					msgStyle = 2;
-				if (message.search("fatal") >= 0)
+				if (message.find("fatal") != std::string::npos)
 					msgStyle = 3;
 				stylesCurrent += std::string(message.length(), static_cast<char>(msgStyle));
 				wEditor.CallString(SCI_ANNOTATIONSETTEXT, sourceLine, msgCurrent.c_str());
@@ -1906,7 +1916,7 @@ void SciTEBase::GoMessage(int dir) {
 	while ((dir == 0) || (lookLine != curLine)) {
 		int startPosLine = wOutput.Call(SCI_POSITIONFROMLINE, lookLine, 0);
 		int lineLength = wOutput.Call(SCI_LINELENGTH, lookLine, 0);
-		char style = acc.StyleAt(startPosLine);
+		int style = acc.StyleAt(startPosLine);
 		if (style != SCE_ERR_DEFAULT &&
 		        style != SCE_ERR_CMD &&
 		        style != SCE_ERR_DIFF_ADDITION &&
@@ -1920,12 +1930,12 @@ void SciTEBase::GoMessage(int dir) {
 			        "error.marker.back", ColourRGB(0xff, 0xff, 0)));
 			wOutput.Call(SCI_MARKERADD, lookLine, 0);
 			wOutput.Call(SCI_SETSEL, startPosLine, startPosLine);
-			SString message = GetRange(wOutput, startPosLine, startPosLine + lineLength);
+			std::string message = GetRangeString(wOutput, startPosLine, startPosLine + lineLength);
 			std::string source;
 			int column;
 			long sourceLine = DecodeMessage(message.c_str(), source, style, column);
 			if (sourceLine >= 0) {
-				GUI::gui_string sourceString = GUI::StringFromUTF8(source.c_str());
+				GUI::gui_string sourceString = GUI::StringFromUTF8(source);
 				FilePath sourcePath = FilePath(sourceString).NormalizePath();
 				if (!filePath.Name().SameNameAs(sourcePath)) {
 					FilePath messagePath;
@@ -1959,13 +1969,13 @@ void SciTEBase::GoMessage(int dir) {
 				if (style == SCE_ERR_CTAG) {
 					//without following focus GetCTag wouldn't work correct
 					WindowSetFocus(wOutput);
-					SString cTag = GetCTag();
+					std::string cTag = GetCTag();
 					if (cTag.length() != 0) {
-						if (cTag.value() > 0) {
+						if (atoi(cTag.c_str()) > 0) {
 							//if tag is linenumber, get line
-							sourceLine = cTag.value() - 1;
+							sourceLine = atoi(cTag.c_str()) - 1;
 						} else {
-							findWhat = cTag.string();
+							findWhat = cTag;
 							FindNext(false);
 							//get linenumber for marker from found position
 							sourceLine = wEditor.Call(SCI_LINEFROMPOSITION, wEditor.Call(SCI_GETCURRENTPOS));
@@ -1974,14 +1984,16 @@ void SciTEBase::GoMessage(int dir) {
 				}
 
 				else if (style == SCE_ERR_DIFF_MESSAGE) {
-					bool isAdd = message.startswith("+++ ");
-					int atLine = lookLine + (isAdd ? 1 : 2); // lines are in this order: ---, +++, @@
-					SString atMessage = GetLine(wOutput, atLine);
-					if (atMessage.startswith("@@ -")) {
-						int atPos = (isAdd
-							? atMessage.search(" +", 7) + 2 // skip "@@ -1,1" and then " +"
-							: 4 // deleted position starts right after "@@ -"
-						);
+					const bool isAdd = message.find("+++ ") == 0;
+					const int atLine = lookLine + (isAdd ? 1 : 2); // lines are in this order: ---, +++, @@
+					std::string atMessage = GetLine(wOutput, atLine);
+					if (atMessage.find("@@ -") == 0) {
+						size_t atPos = 4; // deleted position starts right after "@@ -"
+						if (isAdd) {
+							const size_t linePlace = atMessage.find(" +", 7);
+							if (linePlace != std::string::npos)
+								atPos = linePlace + 2; // skip "@@ -1,1" and then " +"
+						}
 						sourceLine = atol(atMessage.c_str() + atPos) - 1;
 					}
 				}
@@ -2011,8 +2023,8 @@ void SciTEBase::GoMessage(int dir) {
 					//simply move cursor to line, don't do any selection
 					SetSelection(startSourceLine, startSourceLine);
 				}
-				message.substitute('\t', ' ');
-				message.remove("\n");
+				std::replace(message.begin(), message.end(), '\t', ' ');
+				::Remove(message, std::string("\n"));
 				props.Set("CurrentMessage", message.c_str());
 				UpdateStatusBar(false);
 				WindowSetFocus(wEditor);
