@@ -17,13 +17,13 @@
 #include <vector>
 #include <set>
 #include <map>
+#include <algorithm>
 
 #include "Scintilla.h"
 #include "ILexer.h"
 
 #include "GUI.h"
 
-#include "SString.h"
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
@@ -146,28 +146,27 @@ void SciTEBase::DiscoverEOLSetting() {
 
 // Look inside the first line for a #! clue regarding the language
 std::string SciTEBase::DiscoverLanguage() {
-	char buf[64 * 1024];
-	int length = Minimum(LengthDocument(), sizeof(buf)-1);
-	GetRange(wEditor, 0, length, buf);
-	buf[length] = '\0';
+	int length = Minimum(LengthDocument(), 64 * 1024);
+	std::string buf = GetRangeString(wEditor, 0, length);
 	std::string languageOverride = "";
-	SString l1 = ExtractLine(buf, length);
-	if (l1.startswith("<?xml")) {
+	std::string l1 = ExtractLine(buf.c_str(), length);
+	if (StartsWith(l1, "<?xml")) {
 		languageOverride = "xml";
-	} else if (l1.startswith("#!")) {
+	} else if (StartsWith(l1, "#!")) {
 		l1 = l1.substr(2);
-		l1.substitute('\\', ' ');
-		l1.substitute('/', ' ');
-		l1.substitute("\t", " ");
-		l1.substitute("  ", " ");
-		l1.substitute("  ", " ");
-		l1.substitute("  ", " ");
-		l1.remove("\r");
-		l1.remove("\n");
-		if (l1.startswith(" ")) {
+		std::replace(l1.begin(), l1.end(), '\\', ' ');
+		std::replace(l1.begin(), l1.end(), '/', ' ');
+		std::replace(l1.begin(), l1.end(), '\t', ' ');
+		Substitute(l1, "  ", " ");
+		Substitute(l1, "  ", " ");
+		Substitute(l1, "  ", " ");
+		::Remove(l1, std::string("\r"));
+		::Remove(l1, std::string("\n"));
+		if (StartsWith(l1, " ")) {
 			l1 = l1.substr(1);
 		}
-		l1.substitute(' ', '\0');
+		std::replace(l1.begin(), l1.end(), ' ', '\0');
+		l1.append(1, '\0');
 		const char *word = l1.c_str();
 		while (*word) {
 			std::string propShBang("shbang.");
@@ -238,7 +237,7 @@ void SciTEBase::DiscoverIndentSetting() {
 	}
 }
 
-void SciTEBase::OpenFile(long fileSize, bool suppressMessage, bool asynchronous) {
+void SciTEBase::OpenCurrentFile(long fileSize, bool suppressMessage, bool asynchronous) {
 	if (CurrentBuffer()->pFileWorker) {
 		// Already performing an asynchronous load or save so do not restart load
 		if (!suppressMessage) {
@@ -271,7 +270,13 @@ void SciTEBase::OpenFile(long fileSize, bool suppressMessage, bool asynchronous)
 		wEditor.Call(SCI_STYLESETBACK, STYLE_DEFAULT, 0xEEEEEE);
 		wEditor.Call(SCI_SETREADONLY, 1);
 		assert(CurrentBuffer()->pFileWorker == NULL);
-		ILoader *pdocLoad = reinterpret_cast<ILoader *>(wEditor.CallReturnPointer(SCI_CREATELOADER, fileSize + 1000));
+		ILoader *pdocLoad;
+		try {
+			pdocLoad = reinterpret_cast<ILoader *>(wEditor.CallReturnPointer(SCI_CREATELOADER, fileSize + 1000));
+		} catch (...) {
+			wEditor.Call(SCI_SETSTATUS, 0);
+			return;
+		}
 		CurrentBuffer()->pFileWorker = new FileLoader(this, pdocLoad, filePath, fileSize, fp);
 		CurrentBuffer()->pFileWorker->sleepTime = props.GetInt("asynchronous.sleep");
 		PerformOnNewThread(CurrentBuffer()->pFileWorker);
@@ -279,14 +284,22 @@ void SciTEBase::OpenFile(long fileSize, bool suppressMessage, bool asynchronous)
 		wEditor.Call(SCI_ALLOCATE, fileSize + 1000);
 
 		Utf8_16_Read convert;
-		char data[blockSize];
-		size_t lenFile = fread(data, 1, sizeof(data), fp);
-		UniMode umCodingCookie = CodingCookieValue(data, lenFile);
+		std::vector<char> data(blockSize);
+		size_t lenFile = fread(&data[0], 1, data.size(), fp);
+		UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 		while (lenFile > 0) {
-			lenFile = convert.convert(data, lenFile);
+			lenFile = convert.convert(&data[0], lenFile);
 			char *dataBlock = convert.getNewBuf();
 			wEditor.CallString(SCI_ADDTEXT, lenFile, dataBlock);
-			lenFile = fread(data, 1, sizeof(data), fp);
+			lenFile = fread(&data[0], 1, data.size(), fp);
+			if (lenFile == 0) {
+				// Handle case where convert is holding a lead surrogate but no more data
+				size_t lenFileTrail = convert.convert(NULL, lenFile);
+				if (lenFileTrail) {
+					char *dataTrail = convert.getNewBuf();
+					wEditor.CallString(SCI_ADDTEXT, lenFileTrail, dataTrail);
+				}
+			}
 		}
 		fclose(fp);
 		wEditor.Call(SCI_ENDUNDOACTION);
@@ -504,12 +517,12 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 		return false;
 	}
 
-	long size = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
-	if (size > 0) {
+	const long fileSize = absPath.IsUntitled() ? 0 : absPath.GetFileLength();
+	if (fileSize > 0) {
 		// Real file, not empty buffer
 		int maxSize = props.GetInt("max.file.size");
-		if (maxSize > 0 && size > maxSize) {
-			GUI::gui_string sSize = GUI::StringFromInteger(size);
+		if (maxSize > 0 && fileSize > maxSize) {
+			GUI::gui_string sSize = GUI::StringFromInteger(fileSize);
 			GUI::gui_string sMaxSize = GUI::StringFromInteger(maxSize);
 			GUI::gui_string msg = LocaliseMessage("File '^0' is ^1 bytes long,\n"
 			        "larger than the ^2 bytes limit set in the properties.\n"
@@ -538,9 +551,9 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 	SetFileName(absPath);
 
 	propsDiscovered.Clear();
-	SString discoveryScript = props.GetExpanded("command.discover.properties");
+	std::string discoveryScript = props.GetExpandedString("command.discover.properties");
 	if (discoveryScript.length()) {
-		std::string propertiesText = CommandExecute(GUI::StringFromUTF8(discoveryScript.c_str()).c_str(),
+		std::string propertiesText = CommandExecute(GUI::StringFromUTF8(discoveryScript).c_str(),
 			absPath.Directory().AsInternal());
 		if (propertiesText.size()) {
 			propsDiscovered.ReadFromMemory(propertiesText.c_str(), propertiesText.size(), absPath.Directory(), filter, NULL, 0);
@@ -565,9 +578,9 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 			wEditor.Call(SCI_SETUNDOCOLLECTION, 0);
 		}
 
-		asynchronous = (size > props.GetInt("background.open.size", -1)) &&
+		asynchronous = (fileSize > props.GetInt("background.open.size", -1)) &&
 			!(of & (ofPreserveUndo|ofSynchronous));
-		OpenFile(size, of & ofQuiet, asynchronous);
+		OpenCurrentFile(fileSize, of & ofQuiet, asynchronous);
 
 		if (of & ofPreserveUndo) {
 			wEditor.Call(SCI_ENDUNDOACTION);
@@ -591,34 +604,34 @@ bool SciTEBase::Open(FilePath file, OpenFlags of) {
 
 // Returns true if editor should get the focus
 bool SciTEBase::OpenSelected() {
-	SString selName = SelectionFilename();
+	std::string selName = SelectionFilename();
 	if (selName.length() == 0) {
 		WarnUser(warnWrongFile);
 		return false;	// No selection
 	}
 
 #if !defined(GTK)
-	if (selName.startswith("http:") ||
-	        selName.startswith("https:") ||
-	        selName.startswith("ftp:") ||
-	        selName.startswith("ftps:") ||
-	        selName.startswith("news:") ||
-	        selName.startswith("mailto:")) {
-		SString cmd = selName;
+	if (StartsWith(selName, "http:") ||
+		StartsWith(selName, "https:") ||
+		StartsWith(selName, "ftp:") ||
+		StartsWith(selName, "ftps:") ||
+		StartsWith(selName, "news:") ||
+		StartsWith(selName, "mailto:")) {
+		std::string cmd = selName;
 		AddCommand(cmd, "", jobShell);
 		return false;	// Job is done
 	}
 #endif
 
-	if (selName.startswith("file://")) {
-		selName.remove(0, 7);
+	if (StartsWith(selName, "file://")) {
+		selName.erase(0, 7);
 		if (selName[0] == '/' && selName[2] == ':') { // file:///C:/filename.ext
-			selName.remove(0, 1);
+			selName.erase(0, 1);
 		}
 	}
 
-	SString fileNameForExtension = ExtensionFileName();
-	SString openSuffix = props.GetNewExpand("open.suffix.", fileNameForExtension.c_str());
+	std::string fileNameForExtension = ExtensionFileName();
+	std::string openSuffix = props.GetNewExpandString("open.suffix.", fileNameForExtension.c_str());
 	selName += openSuffix;
 
 	if (EqualCaseInsensitive(selName.c_str(), FileNameExt().AsUTF8().c_str()) || EqualCaseInsensitive(selName.c_str(), filePath.AsUTF8().c_str())) {
@@ -626,10 +639,10 @@ bool SciTEBase::OpenSelected() {
 		return true;	// Do not open if it is the current file!
 	}
 
-	SString cTag;
+	std::string cTag;
 	unsigned long lineNumber = 0;
 	if (IsPropertiesFile(filePath) &&
-	        !selName.contains('.')) {
+	        (selName.find('.') == std::string::npos)) {
 		// We are in a properties file and try to open a file without extension,
 		// we suppose we want to open an imported .properties file
 		// So we append the correct extension to open the included file.
@@ -640,17 +653,17 @@ bool SciTEBase::OpenSelected() {
 		// A bit of duplicate work with DecodeMessage, but we don't know
 		// here the format of the line, so we do guess work.
 		// Can't do much for space separated line numbers anyway...
-		int endPath = selName.search("(");
-		if (endPath >= 0) {	// Visual Studio error message: F:\scite\src\SciTEBase.h(312):	bool Exists(
+		size_t endPath = selName.find("(");
+		if (endPath != std::string::npos) {	// Visual Studio error message: F:\scite\src\SciTEBase.h(312):	bool Exists(
 			lineNumber = atol(selName.c_str() + endPath + 1);
 		} else {
-			endPath = selName.search(":", 2);	// Skip Windows' drive separator
-			if (endPath >= 0) {	// grep -n line, perhaps gcc too: F:\scite\src\SciTEBase.h:312:	bool Exists(
+			endPath = selName.find(":", 2);	// Skip Windows' drive separator
+			if (endPath != std::string::npos) {	// grep -n line, perhaps gcc too: F:\scite\src\SciTEBase.h:312:	bool Exists(
 				lineNumber = atol(selName.c_str() + endPath + 1);
 			}
 		}
 		if (lineNumber > 0) {
-			selName.remove(endPath, 0);
+			selName.erase(endPath);
 		}
 
 		// Support the ctags format
@@ -663,13 +676,13 @@ bool SciTEBase::OpenSelected() {
 	FilePath path;
 	// Don't load the path of the current file if the selected
 	// filename is an absolute pathname
-	GUI::gui_string selFN = GUI::StringFromUTF8(selName.c_str());
+	GUI::gui_string selFN = GUI::StringFromUTF8(selName);
 	if (!FilePath(selFN).IsAbsolute()) {
 		path = filePath.Directory();
 		// If not there, look in openpath
 		if (!Exists(path.AsInternal(), selFN.c_str(), NULL)) {
-			GUI::gui_string openPath = GUI::StringFromUTF8(props.GetNewExpand(
-			            "openpath.", fileNameForExtension.c_str()).c_str());
+			GUI::gui_string openPath = GUI::StringFromUTF8(props.GetNewExpandString(
+			            "openpath.", fileNameForExtension.c_str()));
 			while (openPath.length()) {
 				GUI::gui_string tryPath(openPath);
 				size_t sepIndex = tryPath.find(listSepString);
@@ -694,10 +707,10 @@ bool SciTEBase::OpenSelected() {
 			if (lineNumber > 0) {
 				wEditor.Call(SCI_GOTOLINE, lineNumber - 1);
 			} else if (cTag.length() != 0) {
-				if (cTag.value() > 0) {
-					wEditor.Call(SCI_GOTOLINE, cTag.value() - 1);
+				if (atoi(cTag.c_str()) > 0) {
+					wEditor.Call(SCI_GOTOLINE, atoi(cTag.c_str()) - 1);
 				} else {
-					findWhat = cTag.string();
+					findWhat = cTag;
 					FindNext(false);
 				}
 			}
@@ -711,7 +724,7 @@ bool SciTEBase::OpenSelected() {
 
 void SciTEBase::Revert() {
 	RecentFile rf = GetFilePosition();
-	OpenFile(filePath.GetFileLength(), false, false);
+	OpenCurrentFile(filePath.GetFileLength(), false, false);
 	DisplayAround(rf);
 }
 
@@ -827,10 +840,10 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
 		SaveSessionFile(GUI_TEXT(""));
 	}
 
-	// Ensure extender is told about each buffer closing
-	for (int k = 0; k < buffers.lengthVisible; k++) {
-		SetDocumentAt(k);
-		if (extender) {
+	if (extender && extender->NeedsOnClose()) {
+		// Ensure extender is told about each buffer closing
+		for (int k = 0; k < buffers.lengthVisible; k++) {
+			SetDocumentAt(k);
 			extender->OnClose(filePath.AsUTF8().c_str());
 		}
 	}
@@ -842,7 +855,7 @@ SciTEBase::SaveResult SciTEBase::SaveIfUnsureAll() {
 	}
 	// Release all the extra documents
 	for (int j = 0; j < buffers.size; j++) {
-		if (buffers.buffers[j].doc) {
+		if (buffers.buffers[j].doc && !buffers.buffers[j].pFileWorker) {
 			wEditor.Call(SCI_RELEASEDOCUMENT, 0, buffers.buffers[j].doc);
 			buffers.buffers[j].doc = 0;
 		}
@@ -909,9 +922,9 @@ bool SciTEBase::PrepareBufferForSave(FilePath saveName) {
 	bool retVal = false;
 	// Perform clean ups on text before saving
 	wEditor.Call(SCI_BEGINUNDOACTION);
-	SString useStripTrailingSpaces = props.GetNewExpand("strip.trailing.spaces.", ExtensionFileName().c_str());
+	std::string useStripTrailingSpaces = props.GetNewExpandString("strip.trailing.spaces.", ExtensionFileName().c_str());
 	if (useStripTrailingSpaces.length() > 0) {
-		if (useStripTrailingSpaces.value())
+		if (atoi(useStripTrailingSpaces.c_str()))
 			StripTrailingSpaces();
 	} else if (props.GetInt("strip.trailing.spaces"))
 		StripTrailingSpaces();
@@ -957,7 +970,7 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 							static_cast<int>(CurrentBuffer()->unicodeMode)));
 				}
 				convert.setfile(fp);
-				char data[blockSize + 1];
+				std::vector<char> data(blockSize + 1);
 				retVal = true;
 				int grabSize;
 				for (int i = 0; i < lengthDoc; i += grabSize) {
@@ -966,8 +979,8 @@ bool SciTEBase::SaveBuffer(FilePath saveName, SaveFlags sf) {
 						grabSize = blockSize;
 					// Round down so only whole characters retrieved.
 					grabSize = wEditor.Call(SCI_POSITIONBEFORE, i + grabSize + 1) - i;
-					GetRange(wEditor, i, i + grabSize, data);
-					size_t written = convert.fwrite(data, grabSize);
+					GetRange(wEditor, i, i + grabSize, &data[0]);
+					size_t written = convert.fwrite(&data[0], grabSize);
 					if (written == 0) {
 						retVal = false;
 						break;
@@ -1099,7 +1112,7 @@ bool SciTEBase::IsStdinBlocked() {
 
 void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 	Utf8_16_Read convert;
-	char data[blockSize];
+	std::vector<char> data(blockSize);
 
 	/* if stdin is blocked, do not execute this method */
 	if (IsStdinBlocked())
@@ -1112,16 +1125,16 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 		wEditor.Call(SCI_BEGINUNDOACTION);	// Group together clear and insert
 		wEditor.Call(SCI_CLEARALL);
 	}
-	size_t lenFile = fread(data, 1, sizeof(data), stdin);
-	UniMode umCodingCookie = CodingCookieValue(data, lenFile);
+	size_t lenFile = fread(&data[0], 1, data.size(), stdin);
+	UniMode umCodingCookie = CodingCookieValue(&data[0], lenFile);
 	while (lenFile > 0) {
-		lenFile = convert.convert(data, lenFile);
+		lenFile = convert.convert(&data[0], lenFile);
 		if (UseOutputPane) {
 			wOutput.CallString(SCI_ADDTEXT, lenFile, convert.getNewBuf());
 		} else {
 			wEditor.CallString(SCI_ADDTEXT, lenFile, convert.getNewBuf());
 		}
-		lenFile = fread(data, 1, sizeof(data), stdin);
+		lenFile = fread(&data[0], 1, data.size(), stdin);
 	}
 	if (UseOutputPane) {
 		if (props.GetInt("split.vertical") == 0) {
@@ -1164,7 +1177,7 @@ void SciTEBase::OpenFromStdin(bool UseOutputPane) {
 }
 
 void SciTEBase::OpenFilesFromStdin() {
-	char data[blockSize];
+	char data[8 * 1024];
 
 	/* if stdin is blocked, do not execute this method */
 	if (IsStdinBlocked())
@@ -1202,7 +1215,7 @@ class BufferedFile {
 		}
 	}
 public:
-	BufferedFile(FilePath fPath) {
+	explicit BufferedFile(FilePath fPath) {
 		fp = fPath.Open(fileRead);
 		readAll = false;
 		exhausted = fp == NULL;
@@ -1313,7 +1326,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 	FilePathSet files;
 	baseDir.List(directories, files);
 	size_t searchLength = strlen(searchString);
-	SString os;
+	std::string os;
 	for (size_t i = 0; i < files.size(); i ++) {
 		if (jobQueue.Cancelled())
 			return;
@@ -1339,7 +1352,7 @@ void SciTEBase::GrepRecursive(GrepFlags gf, FilePath baseDir, const char *search
 						if (match) {
 							os.append(fPath.AsUTF8().c_str());
 							os.append(":");
-							SString lNumber(fr.LineNumber());
+							std::string lNumber = StdStringFromInteger(fr.LineNumber());
 							os.append(lNumber.c_str());
 							os.append(":");
 							os.append(fr.Original());
@@ -1384,10 +1397,10 @@ void SciTEBase::InternalGrep(GrepFlags gf, const GUI::gui_char *directory, const
 	}
 	GrepRecursive(gf, FilePath(directory), searchString.c_str(), fileTypes);
 	if (!(gf & grepStdOut)) {
-		SString sExitMessage(">");
+		std::string sExitMessage(">");
 		if (jobQueue.TimeCommands()) {
 			sExitMessage += "    Time: ";
-			sExitMessage += SString(commandTime.Duration(), 3);
+			sExitMessage += StdStringFromDouble(commandTime.Duration(), 3);
 		}
 		sExitMessage += "\n";
 		OutputAppendStringSynchronised(sExitMessage.c_str());
