@@ -36,7 +36,6 @@ const GUI::gui_char menuAccessIndicator[] = GUI_TEXT("&");
 
 #endif
 
-#include "SString.h"
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
@@ -91,7 +90,7 @@ void SciTEBase::SetLanguageMenu() {
 #else
 			entry += GUI_TEXT("\t");
 #endif
-			entry += GUI::StringFromUTF8(languageMenu[item].menuKey.c_str());
+			entry += GUI::StringFromUTF8(languageMenu[item].menuKey);
 		}
 		if (entry.size() && entry[0] != '#') {
 			SetMenuItem(menuLanguage, item, itemID, entry.c_str());
@@ -123,13 +122,13 @@ void SciTEBase::ReadGlobalPropFile() {
 		}
 	}
 
-	SString excludes;
-	SString includes;
+	std::string excludes;
+	std::string includes;
 
 	for (int attempt=0; attempt<2; attempt++) {
 
-		SString excludesRead = props.Get("imports.exclude");
-		SString includesRead = props.Get("imports.include");
+		std::string excludesRead = props.GetString("imports.exclude");
+		std::string includesRead = props.GetString("imports.include");
 		if ((attempt > 0) && ((excludesRead == excludes) && (includesRead == includes)))
 			break;
 
@@ -231,6 +230,40 @@ const char *SciTEBase::GetNextPropItem(
 	return pNext;
 }
 
+std::string SciTEBase::StyleString(const char *lang, int style) const {
+	char key[200];
+	sprintf(key, "style.%s.%0d", lang, style);
+	return props.GetExpandedString(key);
+}
+
+StyleDefinition SciTEBase::StyleDefinitionFor(int style) {
+	const std::string languageName = !StartsWith(language, "lpeg_") ? language : "lpeg";
+
+	const std::string ssDefault = StyleString("*", style);
+	std::string ss = StyleString(languageName.c_str(), style);
+
+	if (!subStyleBases.empty()) {
+		const int baseStyle = wEditor.Call(SCI_GETSTYLEFROMSUBSTYLE, style);
+		if (baseStyle != style) {
+			const int primaryStyle = wEditor.Call(SCI_GETPRIMARYSTYLEFROMSTYLE, style);
+			const int distanceSecondary = (style == primaryStyle) ? 0 : wEditor.Call(SCI_DISTANCETOSECONDARYSTYLES);
+			const int primaryBase = baseStyle - distanceSecondary;
+			const int subStylesStart = wEditor.Call(SCI_GETSUBSTYLESSTART, primaryBase);
+			const int subStylesLength = wEditor.Call(SCI_GETSUBSTYLESLENGTH, primaryBase);
+			const int subStyle = style - (subStylesStart + distanceSecondary);
+			if (subStyle < subStylesLength) {
+				char key[200];
+				sprintf(key, "style.%s.%0d.%0d", languageName.c_str(), baseStyle, subStyle + 1);
+				ss = props.GetNewExpandString(key);
+			}
+		}
+	}
+
+	StyleDefinition sd(ssDefault.c_str());
+	sd.ParseStyleDefinition(ss.c_str());
+	return sd;
+}
+
 void SciTEBase::SetOneStyle(GUI::ScintillaWindow &win, int style, const StyleDefinition &sd) {
 	if (sd.specified & StyleDefinition::sdItalics)
 		win.Send(SCI_STYLESETITALIC, style, sd.italics ? 1 : 0);
@@ -263,7 +296,7 @@ void SciTEBase::SetStyleBlock(GUI::ScintillaWindow &win, const char *lang, int s
 		if (style != STYLE_DEFAULT) {
 			char key[200];
 			sprintf(key, "style.%s.%0d", lang, style-start);
-			SString sval = props.GetExpanded(key);
+			std::string sval = props.GetExpandedString(key);
 			if (sval.length()) {
 				SetOneStyle(win, style, sval.c_str());
 			}
@@ -286,30 +319,33 @@ void SciTEBase::SetOneIndicator(GUI::ScintillaWindow &win, int indicator, const
 	win.Call(SCI_INDICSETUNDER, indicator, ind.under);
 }
 
-SString SciTEBase::ExtensionFileName() {
+std::string SciTEBase::ExtensionFileName() const {
 	if (CurrentBuffer()->overrideExtension.length()) {
-		return CurrentBuffer()->overrideExtension.c_str();
+		return CurrentBuffer()->overrideExtension;
 	} else {
 		FilePath name = FileNameExt();
 		if (name.IsSet()) {
-			// Force extension to lower case
-			SString fileNameWithLowerCaseExtension = name.AsUTF8().c_str();
 #if !defined(GTK)
-			const char *extension = strrchr(fileNameWithLowerCaseExtension.c_str(), '.');
-			if (extension) {
-				fileNameWithLowerCaseExtension.lowercase(extension - fileNameWithLowerCaseExtension.c_str());
+			// Force extension to lower case
+			std::string extension = name.Extension().AsUTF8();
+			if (extension.empty()) {
+				return name.AsUTF8();
+			} else {
+				LowerCaseAZ(extension);
+				return name.BaseName().AsUTF8() + "." + extension;
 			}
+#else
+			return name.AsUTF8();
 #endif
-			return fileNameWithLowerCaseExtension;
 		} else {
-			return props.Get("default.file.ext");
+			return props.GetString("default.file.ext");
 		}
 	}
 }
 
 void SciTEBase::ForwardPropertyToEditor(const char *key) {
 	if (props.Exists(key)) {
-		SString value = props.GetExpanded(key);
+		std::string value = props.GetExpandedString(key);
 		wEditor.CallString(SCI_SETPROPERTY,
 						 reinterpret_cast<uptr_t>(key), value.c_str());
 		wOutput.CallString(SCI_SETPROPERTY,
@@ -324,66 +360,32 @@ void SciTEBase::DefineMarker(int marker, int markerType, Colour fore, Colour bac
 	wEditor.Call(SCI_MARKERSETBACKSELECTED, marker, backSelected);
 }
 
-static long FileLength(const char *path) {
-	long len = 0;
-	FILE *fp = fopen(path, "rb");
-	if (fp) {
-		fseek(fp, 0, SEEK_END);
-		len = ftell(fp);
-		fclose(fp);
-	}
-	return len;
-}
-
-void SciTEBase::ReadAPI(const SString &fileNameForExtension) {
-	SString sApiFileNames = props.GetNewExpand("api.",
+void SciTEBase::ReadAPI(const std::string &fileNameForExtension) {
+	std::string sApiFileNames = props.GetNewExpandString("api.",
 	                        fileNameForExtension.c_str());
-	size_t nameLength = sApiFileNames.length();
-	if (nameLength) {
-		sApiFileNames.substitute(';', '\0');
-		const char *apiFileName = sApiFileNames.c_str();
-		const char *nameEnd = apiFileName + nameLength;
+	if (sApiFileNames.length() > 0) {
+		std::vector<std::string> vApiFileNames = StringSplit(sApiFileNames, ';');
+		std::vector<char> data;
 
-		size_t tlen = 0;    // total api length
-
-		// Calculate total length
-		while (apiFileName < nameEnd) {
-			tlen += FileLength(apiFileName);
-			apiFileName += strlen(apiFileName) + 1;
+		// Load files into data
+		for (std::vector<std::string>::iterator it = vApiFileNames.begin(); it != vApiFileNames.end(); ++it) {
+			std::vector<char> contents = FilePath(GUI::StringFromUTF8(*it)).Read();
+			data.insert(data.end(), contents.begin(), contents.end());
 		}
 
-		// Load files
-		if (tlen > 0) {
-			char *buffer = apis.Allocate(static_cast<int>(tlen));
-			if (buffer) {
-				apiFileName = sApiFileNames.c_str();
-				tlen = 0;
-				while (apiFileName < nameEnd) {
-					FILE *fp = fopen(apiFileName, "rb");
-					if (fp) {
-						fseek(fp, 0, SEEK_END);
-						long len = ftell(fp);
-						if (len > 0) {
-							fseek(fp, 0, SEEK_SET);
-							size_t readBytes = fread(buffer + tlen, 1, len, fp);
-							tlen += readBytes;
-						}
-						fclose(fp);
-					}
-					apiFileName += strlen(apiFileName) + 1;
-				}
-				apis.SetFromAllocated();
-			}
+		// Initialise apis
+		if (data.size() > 0) {
+			apis.Set(data);
 		}
 	}
 }
 
-SString SciTEBase::FindLanguageProperty(const char *pattern, const char *defaultValue) {
-	SString key = pattern;
-	key.substitute("*", language.c_str());
-	SString ret = props.GetExpanded(key.c_str());
+std::string SciTEBase::FindLanguageProperty(const char *pattern, const char *defaultValue) {
+	std::string key = pattern;
+	Substitute(key, "*", language.c_str());
+	std::string ret = props.GetExpandedString(key.c_str());
 	if (ret == "")
-		ret = props.GetExpanded(pattern);
+		ret = props.GetExpandedString(pattern);
 	if (ret == "")
 		ret = defaultValue;
 	return ret;
@@ -462,6 +464,7 @@ static const char *propertiesToForward[] = {
 	"lexer.cpp.track.preprocessor",
 	"lexer.cpp.triplequoted.strings",
 	"lexer.cpp.update.preprocessor",
+	"lexer.cpp.verbatim.strings.allow.escapes",
 	"lexer.css.hss.language",
 	"lexer.css.less.language",
 	"lexer.css.scss.language",
@@ -492,6 +495,11 @@ static const char *propertiesToForward[] = {
 	"lexer.tex.comment.process",
 	"lexer.tex.interface.default",
 	"lexer.tex.use.keywords",
+	"lexer.verilog.allupperkeywords",
+	"lexer.verilog.fold.preprocessor.else",
+	"lexer.verilog.portstyling",
+	"lexer.verilog.track.preprocessor",
+	"lexer.verilog.update.preprocessor",
 	"lexer.xml.allow.scripts",
 	"nsis.ignorecase",
 	"nsis.uservars",
@@ -592,15 +600,15 @@ static const char *bookmarkBluegem[] = {
 "    1GdddG1    "
 };
 
-SString SciTEBase::GetFileNameProperty(const char *name) {
+std::string SciTEBase::GetFileNameProperty(const char *name) {
 	std::string namePlusDot = name;
 	namePlusDot.append(".");
-	SString valueForFileName = props.GetNewExpand(namePlusDot.c_str(),
+	std::string valueForFileName = props.GetNewExpandString(namePlusDot.c_str(),
 	        ExtensionFileName().c_str());
 	if (valueForFileName.length() != 0) {
 		return valueForFileName;
 	} else {
-		return props.Get(name);
+		return props.GetString(name);
 	}
 }
 
@@ -608,23 +616,23 @@ void SciTEBase::ReadProperties() {
 	if (extender)
 		extender->Clear();
 
-	SString fileNameForExtension = ExtensionFileName();
+	const std::string fileNameForExtension = ExtensionFileName();
 
-	SString modulePath = props.GetNewExpand("lexerpath.",
+	std::string modulePath = props.GetNewExpandString("lexerpath.",
 	    fileNameForExtension.c_str());
 	if (modulePath.length())
 	    wEditor.CallString(SCI_LOADLEXERLIBRARY, 0, modulePath.c_str());
-	language = props.GetNewExpand("lexer.", fileNameForExtension.c_str());
+	language = props.GetNewExpandString("lexer.", fileNameForExtension.c_str());
 	if (language.length()) {
-		if (language.startswith("script_")) {
+		if (StartsWith(language, "script_")) {
 			wEditor.Call(SCI_SETLEXER, SCLEX_CONTAINER);
-		} else if (language.startswith("lpeg_")) {
-			modulePath = props.GetNewExpand("lexerpath.*.lpeg");
+		} else if (StartsWith(language, "lpeg_")) {
+			modulePath = props.GetNewExpandString("lexerpath.*.lpeg");
 			if (modulePath.length()) {
 				wEditor.CallString(SCI_LOADLEXERLIBRARY, 0, modulePath.c_str());
 				wEditor.CallString(SCI_SETLEXERLANGUAGE, 0, "lpeg");
 				lexLPeg = wEditor.Call(SCI_GETLEXER);
-				const char *lexer = language.c_str() + language.search("_") + 1;
+				const char *lexer = language.c_str() + language.find("_") + 1;
 				wEditor.CallReturnPointer(SCI_PRIVATELEXERCALL, SCI_SETLEXERLANGUAGE,
 					reinterpret_cast<sptr_t>(lexer));
 			}
@@ -639,24 +647,61 @@ void SciTEBase::ReadProperties() {
 
 	lexLanguage = wEditor.Call(SCI_GETLEXER);
 
-	if (language.startswith("script_") || language.startswith("lpeg_"))
+	if (StartsWith(language, "script_") || StartsWith(language, "lpeg_"))
 		wEditor.Call(SCI_SETSTYLEBITS, 8);
 	else
 		wEditor.Call(SCI_SETSTYLEBITS, wEditor.Call(SCI_GETSTYLEBITSNEEDED));
 
 	wOutput.Call(SCI_SETLEXER, SCLEX_ERRORLIST);
 
-	SString kw0 = props.GetNewExpand("keywords.", fileNameForExtension.c_str());
+	const std::string kw0 = props.GetNewExpandString("keywords.", fileNameForExtension.c_str());
 	wEditor.CallString(SCI_SETKEYWORDS, 0, kw0.c_str());
 
 	for (int wl = 1; wl <= KEYWORDSET_MAX; wl++) {
 		std::string kwk = StdStringFromInteger(wl+1);
 		kwk += '.';
 		kwk.insert(0, "keywords");
-		SString kw = props.GetNewExpand(kwk.c_str(), fileNameForExtension.c_str());
+		const std::string kw = props.GetNewExpandString(kwk.c_str(), fileNameForExtension.c_str());
 		wEditor.CallString(SCI_SETKEYWORDS, wl, kw.c_str());
 	}
 
+	subStyleBases.clear();
+	int lenSSB = wEditor.CallString(SCI_GETSUBSTYLEBASES, 0, NULL);
+	if (lenSSB) {
+		wEditor.Call(SCI_FREESUBSTYLES);
+
+		subStyleBases.resize(lenSSB+1);
+		wEditor.CallString(SCI_GETSUBSTYLEBASES, 0, &subStyleBases[0]);
+		subStyleBases.resize(lenSSB);	// Remove NUL
+
+		for (int baseStyle=0;baseStyle<lenSSB;baseStyle++) {
+			//substyles.cpp.11=2
+			std::string ssSubStylesKey = "substyles.";
+			ssSubStylesKey += language;
+			ssSubStylesKey += ".";
+			ssSubStylesKey += StdStringFromInteger(subStyleBases[baseStyle]);
+			std::string ssNumber = props.GetNewExpandString(ssSubStylesKey.c_str());
+			int subStyleIdentifiers = atoi(ssNumber.c_str());
+
+			int subStyleIdentifiersStart = 0;
+			if (subStyleIdentifiers) {
+				subStyleIdentifiersStart = wEditor.Call(SCI_ALLOCATESUBSTYLES, subStyleBases[baseStyle], subStyleIdentifiers);
+				if (subStyleIdentifiersStart < 0)
+					subStyleIdentifiers = 0;
+			}
+			for (int subStyle=0; subStyle<subStyleIdentifiers; subStyle++) {
+				// substylewords.11.1.$(file.patterns.cpp)=CharacterSet LexAccessor SString WordList
+				std::string ssWordsKey = "substylewords.";
+				ssWordsKey += StdStringFromInteger(subStyleBases[baseStyle]);
+				ssWordsKey += ".";
+				ssWordsKey += StdStringFromInteger(subStyle + 1);
+				ssWordsKey += ".";
+				std::string ssWords = props.GetNewExpandString(ssWordsKey.c_str(), fileNameForExtension.c_str());
+				wEditor.CallString(SCI_SETIDENTIFIERS, subStyleIdentifiersStart + subStyle, ssWords.c_str());
+			}
+		}
+	}
+
 	FilePath homepath = GetSciteDefaultHome();
 	props.Set("SciteDefaultHome", homepath.AsUTF8().c_str());
 	homepath = GetSciteUserHome();
@@ -666,15 +711,15 @@ void SciTEBase::ReadProperties() {
 		ForwardPropertyToEditor(propertiesToForward[i]);
 	}
 
-	if (apisFileNames != props.GetNewExpand("api.",	fileNameForExtension.c_str())) {
+	if (apisFileNames != props.GetNewExpandString("api.", fileNameForExtension.c_str())) {
 		apis.Clear();
 		ReadAPI(fileNameForExtension);
-		apisFileNames = props.GetNewExpand("api.", fileNameForExtension.c_str());
+		apisFileNames = props.GetNewExpandString("api.", fileNameForExtension.c_str());
 	}
 
 	props.Set("APIPath", apisFileNames.c_str());
 
-	FilePath fileAbbrev = GUI::StringFromUTF8(props.GetNewExpand("abbreviations.", fileNameForExtension.c_str()).c_str());
+	FilePath fileAbbrev = GUI::StringFromUTF8(props.GetNewExpandString("abbreviations.", fileNameForExtension.c_str()));
 	if (!fileAbbrev.IsSet())
 		fileAbbrev = GetAbbrevPropertiesFileName();
 	if (!pathAbbreviations.SameNameAs(fileAbbrev)) {
@@ -700,13 +745,19 @@ void SciTEBase::ReadProperties() {
 	characterSet = props.GetInt("character.set", SC_CHARSET_DEFAULT);
 
 #ifdef __unix__
-	SString localeCType = props.Get("LC_CTYPE");
+	const std::string localeCType = props.GetString("LC_CTYPE");
 	if (localeCType.length())
 		setlocale(LC_CTYPE, localeCType.c_str());
 	else
 		setlocale(LC_CTYPE, "C");
 #endif
 
+	std::string imeInteraction = props.GetString("ime.interaction");
+	if (imeInteraction.length()) {
+		CallChildren(SCI_SETIMEINTERACTION, props.GetInt("ime.interaction", SC_IME_WINDOWED));
+	}
+	imeAutoComplete = props.GetInt("ime.autocomplete", 0) == 1;
+
 	wrapStyle = props.GetInt("wrap.style", SC_WRAP_WORD);
 
 	CallChildren(SCI_SETCARETFORE,
@@ -741,17 +792,17 @@ void SciTEBase::ReadProperties() {
 
 	closeFind = props.GetInt("find.close.on.find", 1);
 
-	SString controlCharSymbol = props.Get("control.char.symbol");
+	const std::string controlCharSymbol = props.GetString("control.char.symbol");
 	if (controlCharSymbol.length()) {
 		wEditor.Call(SCI_SETCONTROLCHARSYMBOL, static_cast<unsigned char>(controlCharSymbol[0]));
 	} else {
 		wEditor.Call(SCI_SETCONTROLCHARSYMBOL, 0);
 	}
 
-	SString caretPeriod = props.Get("caret.period");
+	const std::string caretPeriod = props.GetString("caret.period");
 	if (caretPeriod.length()) {
-		wEditor.Call(SCI_SETCARETPERIOD, caretPeriod.value());
-		wOutput.Call(SCI_SETCARETPERIOD, caretPeriod.value());
+		wEditor.Call(SCI_SETCARETPERIOD, atoi(caretPeriod.c_str()));
+		wOutput.Call(SCI_SETCARETPERIOD, atoi(caretPeriod.c_str()));
 	}
 
 	int caretSlop = props.GetInt("caret.policy.xslop", 1) ? CARET_SLOP : 0;
@@ -838,7 +889,7 @@ void SciTEBase::ReadProperties() {
 	bracesStyle = props.GetInt(bracesStyleKey, 0);
 
 	char key[200];
-	SString sval;
+	std::string sval;
 
 	sval = FindLanguageProperty("calltip.*.ignorecase");
 	callTipIgnoreCase = sval == "1";
@@ -854,24 +905,24 @@ void SciTEBase::ReadProperties() {
 	calltipEndDefinition = FindLanguageProperty("calltip.*.end.definition");
 
 	sprintf(key, "autocomplete.%s.start.characters", language.c_str());
-	autoCompleteStartCharacters = props.GetExpanded(key);
+	autoCompleteStartCharacters = props.GetExpandedString(key);
 	if (autoCompleteStartCharacters == "")
-		autoCompleteStartCharacters = props.GetExpanded("autocomplete.*.start.characters");
+		autoCompleteStartCharacters = props.GetExpandedString("autocomplete.*.start.characters");
 	// "" is a quite reasonable value for this setting
 
 	sprintf(key, "autocomplete.%s.fillups", language.c_str());
-	autoCompleteFillUpCharacters = props.GetExpanded(key);
+	autoCompleteFillUpCharacters = props.GetExpandedString(key);
 	if (autoCompleteFillUpCharacters == "")
 		autoCompleteFillUpCharacters =
-			props.GetExpanded("autocomplete.*.fillups");
+			props.GetExpandedString("autocomplete.*.fillups");
 	wEditor.CallString(SCI_AUTOCSETFILLUPS, 0,
 		autoCompleteFillUpCharacters.c_str());
 
 	sprintf(key, "autocomplete.%s.ignorecase", "*");
-	sval = props.GetNewExpand(key);
+	sval = props.GetNewExpandString(key);
 	autoCompleteIgnoreCase = sval == "1";
 	sprintf(key, "autocomplete.%s.ignorecase", language.c_str());
-	sval = props.GetNewExpand(key);
+	sval = props.GetNewExpandString(key);
 	if (sval != "")
 		autoCompleteIgnoreCase = sval == "1";
 	wEditor.Call(SCI_AUTOCSETIGNORECASE, autoCompleteIgnoreCase ? 1 : 0);
@@ -907,11 +958,11 @@ void SciTEBase::ReadProperties() {
 		marginWidth = marginWidthDefault;
 	wEditor.Call(SCI_SETMARGINWIDTHN, 1, margin ? marginWidth : 0);
 
-	SString lineMarginProp = props.Get("line.margin.width");
-	lineNumbersWidth = lineMarginProp.value();
+	const std::string lineMarginProp = props.GetString("line.margin.width");
+	lineNumbersWidth = atoi(lineMarginProp.c_str());
 	if (lineNumbersWidth == 0)
 		lineNumbersWidth = lineNumbersWidthDefault;
-	lineNumbersExpand = lineMarginProp.contains('+');
+	lineNumbersExpand = lineMarginProp.find('+') != std::string::npos;
 
 	SetLineNumberWidth();
 
@@ -932,20 +983,20 @@ void SciTEBase::ReadProperties() {
 	bracesSloppy = props.GetInt("braces.sloppy");
 
 	wEditor.Call(SCI_SETCHARSDEFAULT);
-	wordCharacters = props.GetNewExpand("word.characters.", fileNameForExtension.c_str());
+	wordCharacters = props.GetNewExpandString("word.characters.", fileNameForExtension.c_str());
 	if (wordCharacters.length()) {
 		wEditor.CallString(SCI_SETWORDCHARS, 0, wordCharacters.c_str());
 	} else {
 		wordCharacters = "_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
 	}
 
-	whitespaceCharacters = props.GetNewExpand("whitespace.characters.", fileNameForExtension.c_str());
+	whitespaceCharacters = props.GetNewExpandString("whitespace.characters.", fileNameForExtension.c_str());
 	if (whitespaceCharacters.length()) {
 		wEditor.CallString(SCI_SETWHITESPACECHARS, 0, whitespaceCharacters.c_str());
 	}
 
-	SString viewIndentExamine = GetFileNameProperty("view.indentation.examine");
-	indentExamine = viewIndentExamine.length() ? viewIndentExamine.value() : SC_IV_REAL;
+	const std::string viewIndentExamine = GetFileNameProperty("view.indentation.examine");
+	indentExamine = viewIndentExamine.length() ? (atoi(viewIndentExamine.c_str())) : SC_IV_REAL;
 	wEditor.Call(SCI_SETINDENTATIONGUIDES, props.GetInt("view.indentation.guides") ?
 		indentExamine : SC_IV_NONE);
 
@@ -956,10 +1007,10 @@ void SciTEBase::ReadProperties() {
 
 	indentOpening = props.GetInt("indent.opening");
 	indentClosing = props.GetInt("indent.closing");
-	indentMaintain = props.GetNewExpand("indent.maintain.", fileNameForExtension.c_str()).value();
+	indentMaintain = atoi(props.GetNewExpandString("indent.maintain.", fileNameForExtension.c_str()).c_str());
 
-	SString lookback = props.GetNewExpand("statement.lookback.", fileNameForExtension.c_str());
-	statementLookback = lookback.value();
+	const std::string lookback = props.GetNewExpandString("statement.lookback.", fileNameForExtension.c_str());
+	statementLookback = atoi(lookback.c_str());
 	statementIndent = GetStyleAndWords("statement.indent.");
 	statementEnd = GetStyleAndWords("statement.end.");
 	blockStart = GetStyleAndWords("block.start.");
@@ -973,19 +1024,18 @@ void SciTEBase::ReadProperties() {
 		{"preprocessor.middle.", ppcMiddle},
 		{"preprocessor.end.", ppcEnd},
 	};
-	SString list;
-	list = props.GetNewExpand("preprocessor.symbol.", fileNameForExtension.c_str());
-	preprocessorSymbol = list[0];
+	std::string list = props.GetNewExpandString("preprocessor.symbol.", fileNameForExtension.c_str());
+	preprocessorSymbol = list.empty() ? 0 : list[0];
 	preprocOfString.clear();
 	for (size_t iPreproc = 0; iPreproc < ELEMENTS(propToPPC); iPreproc++) {
-		list = props.GetNewExpand(propToPPC[iPreproc].propName, fileNameForExtension.c_str());
-		std::vector<std::string> words = StringSplit(std::string(list.c_str()), ' ');
+		list = props.GetNewExpandString(propToPPC[iPreproc].propName, fileNameForExtension.c_str());
+		std::vector<std::string> words = StringSplit(list, ' ');
 		for (std::vector<std::string>::iterator it = words.begin(); it != words.end(); ++it) {
 			preprocOfString[*it] = propToPPC[iPreproc].ppc;
 		}
 	}
 
-	memFiles.AppendList(props.GetNewExpand("find.files").c_str());
+	memFiles.AppendList(props.GetNewExpandString("find.files").c_str());
 
 	wEditor.Call(SCI_SETWRAPVISUALFLAGS, props.GetInt("wrap.visual.flags"));
 	wEditor.Call(SCI_SETWRAPVISUALFLAGSLOCATION, props.GetInt("wrap.visual.flags.location"));
@@ -1156,11 +1206,11 @@ void SciTEBase::ReadProperties() {
 		ColourOfProperty(props, "bookmark.back", ColourRGB(0xe2, 0x40, 0x40)));
 	wEditor.Call(SCI_MARKERSETALPHA, markerBookmark,
 		props.GetInt("bookmark.alpha", SC_ALPHA_NOALPHA));
-	SString bookMarkXPM = props.Get("bookmark.pixmap");
+	const std::string bookMarkXPM = props.GetString("bookmark.pixmap");
 	if (bookMarkXPM.length()) {
 		wEditor.CallString(SCI_MARKERDEFINEPIXMAP, markerBookmark,
 			bookMarkXPM.c_str());
-	} else if (props.Get("bookmark.fore").length()) {
+	} else if (props.GetString("bookmark.fore").length()) {
 		wEditor.Call(SCI_MARKERDEFINE, markerBookmark, SC_MARK_BOOKMARK);
 	} else {
 		// No bookmark.fore setting so display default pixmap.
@@ -1189,7 +1239,7 @@ void SciTEBase::ReadProperties() {
 
 	currentWordHighlight.isEnabled = props.GetInt("highlight.current.word", 0) == 1;
 	if (currentWordHighlight.isEnabled) {
-		SString highlightCurrentWordIndicatorString = props.Get("highlight.current.word.indicator");
+		const std::string highlightCurrentWordIndicatorString = props.GetString("highlight.current.word.indicator");
 		IndicatorDefinition highlightCurrentWordIndicator(highlightCurrentWordIndicatorString.c_str());
 		if (highlightCurrentWordIndicatorString.length() == 0) {
 			highlightCurrentWordIndicator.style = INDIC_ROUNDBOX;
@@ -1214,7 +1264,7 @@ void SciTEBase::ReadProperties() {
 
 		// Check for an extension script
 		GUI::gui_string extensionFile = GUI::StringFromUTF8(
-			props.GetNewExpand("extension.", fileNameForExtension.c_str()).c_str());
+			props.GetNewExpandString("extension.", fileNameForExtension.c_str()));
 		if (extensionFile.length()) {
 			// find file in local directory
 			FilePath docDir = filePath.Directory();
@@ -1244,7 +1294,6 @@ void SciTEBase::ReadProperties() {
 
 void SciTEBase::ReadFontProperties() {
 	char key[200];
-	SString sval;
 	const char *languageName = language.c_str();
 
 	if (lexLanguage == lexLPeg) {
@@ -1269,12 +1318,12 @@ void SciTEBase::ReadFontProperties() {
 	wOutput.Call(SCI_STYLERESETDEFAULT, 0, 0);
 
 	sprintf(key, "style.%s.%0d", "*", STYLE_DEFAULT);
-	sval = props.GetNewExpand(key);
+	std::string sval = props.GetNewExpandString(key);
 	SetOneStyle(wEditor, STYLE_DEFAULT, sval.c_str());
 	SetOneStyle(wOutput, STYLE_DEFAULT, sval.c_str());
 
 	sprintf(key, "style.%s.%0d", languageName, STYLE_DEFAULT);
-	sval = props.GetNewExpand(key);
+	sval = props.GetNewExpandString(key);
 	SetOneStyle(wEditor, STYLE_DEFAULT, sval.c_str());
 
 	wEditor.Call(SCI_STYLECLEARALL, 0, 0);
@@ -1287,6 +1336,20 @@ void SciTEBase::ReadFontProperties() {
 		SetStyleBlock(wEditor, "error", diagnosticStyleStart, diagnosticStyleStart+diagnosticStyles-1);
 	}
 
+	int diffToSecondary = static_cast<int>(wEditor.Call(SCI_DISTANCETOSECONDARYSTYLES));
+	for (unsigned int baseStyle=0; baseStyle<subStyleBases.size(); baseStyle++) {
+		int subStylesStart = wEditor.Call(SCI_GETSUBSTYLESSTART, subStyleBases[baseStyle]);
+		int subStylesLength = wEditor.Call(SCI_GETSUBSTYLESLENGTH, subStyleBases[baseStyle]);
+		for (int subStyle=0; subStyle<subStylesLength; subStyle++) {
+			for (int active=0; active<(diffToSecondary?2:1); active++) {
+				int activity = active * diffToSecondary;
+				sprintf(key, "style.%s.%0d.%0d", languageName, subStyleBases[baseStyle] + activity, subStyle+1);
+				sval = props.GetNewExpandString(key);
+				SetOneStyle(wEditor, subStylesStart + subStyle + activity, sval.c_str());
+			}
+		}
+	}
+
 	// Turn grey while loading
 	if (CurrentBuffer()->lifeState == Buffer::reading)
 		wEditor.Call(SCI_STYLESETBACK, STYLE_DEFAULT, 0xEEEEEE);
@@ -1294,7 +1357,7 @@ void SciTEBase::ReadFontProperties() {
 	wOutput.Call(SCI_STYLECLEARALL, 0, 0);
 
 	sprintf(key, "style.%s.%0d", "errorlist", STYLE_DEFAULT);
-	sval = props.GetNewExpand(key);
+	sval = props.GetNewExpandString(key);
 	SetOneStyle(wOutput, STYLE_DEFAULT, sval.c_str());
 
 	wOutput.Call(SCI_STYLECLEARALL, 0, 0);
@@ -1303,7 +1366,7 @@ void SciTEBase::ReadFontProperties() {
 	SetStyleFor(wOutput, "errorlist");
 
 	if (CurrentBuffer()->useMonoFont) {
-		sval = props.GetExpanded("font.monospace");
+		sval = props.GetExpandedString("font.monospace");
 		StyleDefinition sd(sval.c_str());
 		for (int style = 0; style <= STYLE_MAX; style++) {
 			if (style != STYLE_LINENUMBER) {
@@ -1341,19 +1404,19 @@ void SciTEBase::SetPropertiesInitial() {
 }
 
 GUI::gui_string Localization::Text(const char *s, bool retainIfNotFound) {
-	const char *utfEllipse = "\xe2\x80\xa6";	// A UTF-8 ellipse
-	SString translation = s;
-	int ellipseIndicator = translation.remove("...");
-	int utfEllipseIndicator = translation.remove(utfEllipse);
-	char menuAccessIndicatorChar[2] = "!";
-	menuAccessIndicatorChar[0] = static_cast<char>(menuAccessIndicator[0]);
-	int accessKeyPresent = translation.remove(menuAccessIndicatorChar);
-	translation.lowercase();
-	translation.substitute("\n", "\\n");
-	translation = Get(translation.c_str());
+	const std::string sEllipse("...");	// An ASCII ellipse
+	const std::string utfEllipse("\xe2\x80\xa6");	// A UTF-8 ellipse
+	std::string translation = s;
+	const int ellipseIndicator = Remove(translation, sEllipse);
+	const int utfEllipseIndicator = Remove(translation, utfEllipse);
+	const std::string menuAccessIndicatorChar(1, static_cast<char>(menuAccessIndicator[0]));
+	const int accessKeyPresent = Remove(translation, menuAccessIndicatorChar);
+	LowerCaseAZ(translation);
+	Substitute(translation, "\n", "\\n");
+	translation = GetString(translation.c_str());
 	if (translation.length()) {
 		if (ellipseIndicator)
-			translation += "...";
+			translation += sEllipse;
 		if (utfEllipseIndicator)
 			translation += utfEllipse;
 		if (0 == accessKeyPresent) {
@@ -1361,23 +1424,23 @@ GUI::gui_string Localization::Text(const char *s, bool retainIfNotFound) {
 			// Following codes are required because accelerator is not always
 			// part of alphabetical word in several language. In these cases,
 			// accelerator is written like "(&O)".
-			int posOpenParenAnd = translation.search("(&");
-			if (posOpenParenAnd > 0 && translation.search(")", posOpenParenAnd) == posOpenParenAnd+3) {
-				translation.remove(posOpenParenAnd, 4);
+			const size_t posOpenParenAnd = translation.find("(&");
+			if ((posOpenParenAnd != std::string::npos) && (translation.find(")", posOpenParenAnd) == posOpenParenAnd + 3)) {
+				translation.erase(posOpenParenAnd, 4);
 			} else {
-				translation.remove("&");
+				Remove(translation, std::string("&"));
 			}
 #else
-			translation.remove("&");
+			Remove(translation, std::string("&"));
 #endif
 		}
-		translation.substitute("&", menuAccessIndicatorChar);
-		translation.substitute("\\n", "\n");
+		Substitute(translation, "&", menuAccessIndicatorChar);
+		Substitute(translation, "\\n", "\n");
 	} else {
 		translation = missing;
 	}
 	if ((translation.length() > 0) || !retainIfNotFound) {
-		return GUI::StringFromUTF8(translation.c_str());
+		return GUI::StringFromUTF8(translation);
 	}
 	return GUI::StringFromUTF8(s);
 }
@@ -1396,14 +1459,14 @@ GUI::gui_string SciTEBase::LocaliseMessage(const char *s, const GUI::gui_char *p
 void SciTEBase::ReadLocalization() {
 	localiser.Clear();
 	GUI::gui_string title = GUI_TEXT("locale.properties");
-	SString localeProps = props.GetExpanded("locale.properties");
+	const std::string localeProps = props.GetExpandedString("locale.properties");
 	if (localeProps.length()) {
-		title = GUI::StringFromUTF8(localeProps.c_str());
+		title = GUI::StringFromUTF8(localeProps);
 	}
 	FilePath propdir = GetSciteDefaultHome();
 	FilePath localePath(propdir, title);
 	localiser.Read(localePath, propdir, filter, &importFiles, 0);
-	localiser.SetMissing(props.Get("translation.missing"));
+	localiser.SetMissing(props.GetString("translation.missing"));
 	localiser.read = true;
 }
 
@@ -1431,8 +1494,8 @@ void SciTEBase::ReadPropertiesInitial() {
 	wEditor.Call(SCI_SETWRAPMODE, wrap ? wrapStyle : SC_WRAP_NONE);
 	wOutput.Call(SCI_SETWRAPMODE, wrapOutput ? wrapStyle : SC_WRAP_NONE);
 
-	SString menuLanguageProp = props.GetNewExpand("menu.language");
-	menuLanguageProp.substitute('|', '\0');
+	std::string menuLanguageProp = props.GetNewExpandString("menu.language");
+	std::replace(menuLanguageProp.begin(), menuLanguageProp.end(), '|', '\0');
 	const char *sMenuLanguage = menuLanguageProp.c_str();
 	while (*sMenuLanguage) {
 		LanguageMenuItem lmi;
@@ -1447,11 +1510,10 @@ void SciTEBase::ReadPropertiesInitial() {
 	SetLanguageMenu();
 
 	// load the user defined short cut props
-	SString shortCutProp = props.GetNewExpand("user.shortcuts");
+	std::string shortCutProp = props.GetNewExpandString("user.shortcuts");
 	if (shortCutProp.length()) {
-		size_t pipes = std::count(shortCutProp.c_str(),
-			shortCutProp.c_str()+shortCutProp.length(), '|');
-		shortCutProp.substitute('|', '\0');
+		size_t pipes = std::count(shortCutProp.begin(), shortCutProp.end(), '|');
+		std::replace(shortCutProp.begin(), shortCutProp.end(), '|', '\0');
 		const char *sShortCutProp = shortCutProp.c_str();
 		for (size_t item = 0; item < pipes/2; item++) {
 			ShortcutItem sci;
@@ -1525,7 +1587,7 @@ void SciTEBase::OpenProperties(int propsFile) {
 		Open(propfile, ofQuiet);
 		break;
 	case IDM_OPENLUAEXTERNALFILE: {
-			GUI::gui_string extlua = GUI::StringFromUTF8(props.GetExpanded("ext.lua.startup.script").c_str());
+			GUI::gui_string extlua = GUI::StringFromUTF8(props.GetExpandedString("ext.lua.startup.script"));
 			if (extlua.length()) {
 				Open(extlua.c_str(), ofQuiet);
 			}
@@ -1543,11 +1605,11 @@ void SciTEBase::OpenProperties(int propsFile) {
 }
 
 // return the int value of the command name passed in.
-int SciTEBase::GetMenuCommandAsInt(SString commandName) {
+int SciTEBase::GetMenuCommandAsInt(std::string commandName) {
 	int i = IFaceTable::FindConstant(commandName.c_str());
 	if (i != -1) {
 		return IFaceTable::constants[i].value;
 	}
 	// Otherwise we might have entered a number as command to access a "SCI_" command
-	return commandName.value();
+	return atoi(commandName.c_str());
 }
