@@ -10,6 +10,7 @@
 
 #include <string>
 #include <vector>
+#include <sstream>
 #include <algorithm>
 #include <functional>
 
@@ -23,11 +24,22 @@ bool StartsWith(GUI::gui_string const &s, GUI::gui_string const &start) {
 		(std::equal(s.begin(), s.begin() + start.size(), start.begin()));
 }
 
+bool StartsWith(std::string const &s, const char *start) {
+	const size_t startSize = strlen(start);
+	std::string sStart(start);
+	return (s.size() >= startSize) &&
+		(std::equal(s.begin(), s.begin() + startSize, sStart.begin()));
+}
+
 bool EndsWith(GUI::gui_string const &s, GUI::gui_string const &end) {
 	return (s.size() >= end.size()) &&
 		(std::equal(s.begin() + s.size() - end.size(), s.end(), end.begin()));
 }
 
+bool Contains(std::string const &s, char ch) {
+	return s.find(ch) != std::string::npos;
+}
+
 int Substitute(std::wstring &s, const std::wstring &sFind, const std::wstring &sReplace) {
 	int c = 0;
 	size_t lenFind = sFind.size();
@@ -54,12 +66,33 @@ int Substitute(std::string &s, const std::string &sFind, const std::string &sRep
 	return c;
 }
 
+bool RemoveStringOnce(std::string &s, const char *marker) {
+	size_t modText = s.find(marker);
+	if (modText != std::string::npos) {
+		s.erase(modText, strlen(marker));
+		return true;
+	}
+	return false;
+}
+
 std::string StdStringFromInteger(int i) {
 	char number[32];
 	sprintf(number, "%0d", i);
 	return std::string(number);
 }
 
+std::string StdStringFromSizeT(size_t i) {
+	std::ostringstream strstrm;
+	strstrm << i;
+	return strstrm.str();
+}
+
+std::string StdStringFromDouble(double d, int precision) {
+	char number[32];
+	sprintf(number, "%.*f", precision, d);
+	return std::string(number);
+}
+
 static int LowerCaseAZ(int c) {
 	if (c >= 'A' && c <= 'Z') {
 		return c - 'A' + 'a';
@@ -72,6 +105,65 @@ void LowerCaseAZ(std::string &s) {
 	std::transform(s.begin(), s.end(), s.begin(), std::ptr_fun<int, int>(LowerCaseAZ));
 }
 
+int CompareNoCase(const char *a, const char *b) {
+	while (*a && *b) {
+		if (*a != *b) {
+			char upperA = MakeUpperCase(*a);
+			char upperB = MakeUpperCase(*b);
+			if (upperA != upperB)
+				return upperA - upperB;
+		}
+		a++;
+		b++;
+	}
+	// Either *a or *b is nul
+	return *a - *b;
+}
+
+bool EqualCaseInsensitive(const char *a, const char *b) {
+	return 0 == CompareNoCase(a, b);
+}
+
+bool isprefix(const char *target, const char *prefix) {
+	while (*target && *prefix) {
+		if (*target != *prefix)
+			return false;
+		target++;
+		prefix++;
+	}
+	if (*prefix)
+		return false;
+	else
+		return true;
+}
+
+unsigned int UTF32Character(const unsigned char *utf8) {
+	unsigned char ch = utf8[0];
+	unsigned int u32Char;
+	if (ch < 0x80) {
+		u32Char = ch;
+	} else if (ch < 0x80 + 0x40 + 0x20) {
+		u32Char = (ch & 0x1F) << 6;
+		ch = utf8[1];
+		u32Char += ch & 0x7F;
+	} else if (ch < 0x80 + 0x40 + 0x20 + 0x10) {
+		u32Char = (ch & 0xF) << 12;
+		ch = utf8[1];
+		u32Char += (ch & 0x7F) << 6;
+		ch = utf8[2];
+		u32Char += ch & 0x7F;
+	} else {
+		u32Char = (ch & 0x7) << 18;
+		ch = utf8[1];
+		u32Char += (ch & 0x3F) << 12;
+		ch = utf8[2];
+		u32Char += (ch & 0x3F) << 6;
+		ch = utf8[3];
+		u32Char += ch & 0x3F;
+	}
+	return u32Char;
+}
+
 /**
  * Convert a string into C string literal form using \a, \b, \f, \n, \r, \t, \v, and \ooo.
  * The return value is a newly allocated character array containing the result.
