@@ -15,7 +15,6 @@
 #include "Scintilla.h"
 
 #include "GUI.h"
-#include "SString.h"
 #include "StringHelpers.h"
 #include "StyleDefinition.h"
 
@@ -32,7 +31,8 @@ bool StyleDefinition::ParseStyleDefinition(const char *definition) {
 	if (definition == NULL || *definition == '\0') {
 		return false;
 	}
-	char *val = StringDup(definition);
+	std::vector<char> valHolder(definition, definition + strlen(definition)+1);
+	char *val = &valHolder[0];
 	char *opt = val;
 	while (opt) {
 		// Find attribute separator
@@ -132,7 +132,6 @@ bool StyleDefinition::ParseStyleDefinition(const char *definition) {
 		else
 			opt = 0;
 	}
-	delete []val;
 	return true;
 }
 
@@ -207,6 +206,8 @@ bool IndicatorDefinition::ParseIndicatorDefinition(const char *definition) {
 		{ "dotbox", INDIC_DOTBOX },
 		{ "squigglepixmap", INDIC_SQUIGGLEPIXMAP },
 		{ "compositionthick", INDIC_COMPOSITIONTHICK },
+		{ "compositionthin", INDIC_COMPOSITIONTHIN },
+		{ "fullbox", INDIC_FULLBOX },
 	};
 
 	std::string val(definition);
