@@ -31,7 +31,7 @@ protected:
 	bool InternalIsLeadByte(char ch) const;
 	void Fill(int position);
 public:
-	TextReader(GUI::ScintillaWindow &sw_);
+	explicit TextReader(GUI::ScintillaWindow &sw_);
 	char operator[](int position) {
 		if (position < startPos || position >= endPos) {
 			Fill(position);
@@ -56,7 +56,7 @@ public:
 		codePage = codePage_;
 	}
 	bool Match(int pos, const char *s);
-	char StyleAt(int position);
+	int StyleAt(int position);
 	int GetLine(int position);
 	int LineStart(int line);
 	int LevelAt(int line);
@@ -74,7 +74,7 @@ protected:
 	int validLen;
 	unsigned int startSeg;
 public:
-	StyleWriter(GUI::ScintillaWindow &sw_);
+	explicit StyleWriter(GUI::ScintillaWindow &sw_);
 	void Flush();
 	int SetLineState(int line, int state);
 
