@@ -15,7 +15,7 @@
 #include <map>
 
 #undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
+#define _WIN32_WINNT  0x0602
 #include <windows.h>
 #include <commctrl.h>
 
@@ -23,7 +23,6 @@
 #include "ILexer.h"
 
 #include "GUI.h"
-#include "SString.h"
 #include "StringList.h"
 #include "StringHelpers.h"
 #include "FilePath.h"
@@ -46,13 +45,17 @@ static bool startedByDirector = false;
 static bool shuttingDown = false;
 unsigned int SDI = 0;
 
+static bool HasConnection() {
+	return (wDirector != 0) || (wCorrespondent != 0);
+}
+
 static void SendDirector(const char *verb, const char *arg = 0) {
-	if ((wDirector != 0) || (wCorrespondent != 0)) {
+	if (HasConnection()) {
 		HWND wDestination = wCorrespondent;
-		SString addressedMessage;
+		std::string addressedMessage;
 		if (wDestination) {
 			addressedMessage += ":";
-			SString address(reinterpret_cast<size_t>(wDestination));
+			std::string address = StdStringFromSizeT(reinterpret_cast<size_t>(wDestination));
 			addressedMessage += address;
 			addressedMessage += ":";
 		} else {
@@ -78,22 +81,30 @@ static void SendDirector(const char *verb, const char *arg = 0) {
 }
 
 static void SendDirector(const char *verb, sptr_t arg) {
-	SString s(static_cast<size_t>(arg));
+	std::string s = StdStringFromSizeT(static_cast<size_t>(arg));
 	::SendDirector(verb, s.c_str());
 }
 
+static HWND HwndFromString(const char *s) {
+#ifdef _WIN64
+	return reinterpret_cast<HWND>(strtoull(s, NULL, 10));
+#else
+	return reinterpret_cast<HWND>(atoi(s));
+#endif
+}
+
 static void CheckEnvironment(ExtensionAPI *phost) {
 	if (phost && !shuttingDown) {
 		if (!wDirector) {
 			std::string director = phost->Property("director.hwnd");
 			if (director.length() > 0) {
 				startedByDirector = true;
-				wDirector = reinterpret_cast<HWND>(atoi(director.c_str()));
+				wDirector = HwndFromString(director.c_str());
 				// Director is just seen so identify this to it
 				::SendDirector("identity", reinterpret_cast<sptr_t>(wReceiver));
 			}
 		}
-		SString sReceiver(reinterpret_cast<size_t>(wReceiver));
+		std::string sReceiver = StdStringFromSizeT(reinterpret_cast<size_t>(wReceiver));
 		phost->SetProperty("WindowID", sReceiver.c_str());
 	}
 }
@@ -216,6 +227,11 @@ bool DirectorExtension::OnClose(const char *path) {
 	return false;
 }
 
+bool DirectorExtension::NeedsOnClose() {
+	CheckEnvironment(host);
+	return HasConnection();
+}
+
 bool DirectorExtension::OnChar(char) {
 	return false;
 }
@@ -271,7 +287,7 @@ void DirectorExtension::HandleStringMessage(const char *message) {
 	// be set to zero before time.
 	StringList wlMessage(true);
 	wlMessage.Set(message);
-	for (int i = 0; i < wlMessage.len; i++) {
+	for (size_t i = 0; i < wlMessage.Length(); i++) {
 		// Message format is [:return address:]command:argument
 		char *cmd = wlMessage[i];
 		if (*cmd == ':') {
@@ -279,19 +295,21 @@ void DirectorExtension::HandleStringMessage(const char *message) {
 			char *colon = strchr(cmd + 1, ':');
 			if (colon) {
 				*colon = '\0';
-				wCorrespondent = reinterpret_cast<HWND>(atoi(cmd + 1));
+				wCorrespondent = HwndFromString(cmd + 1);
 				cmd = colon + 1;
 			}
 		}
 		if (isprefix(cmd, "identity:")) {
 			char *arg = strchr(cmd, ':');
 			if (arg)
-				wDirector = reinterpret_cast<HWND>(atoi(arg + 1));
+				wDirector = HwndFromString(arg + 1);
 		} else if (isprefix(cmd, "closing:")) {
 			wDirector = 0;
 			if (startedByDirector) {
 				shuttingDown = true;
-				host->ShutDown();
+				if (host) {
+					host->ShutDown();
+				}
 				shuttingDown = false;
 			}
 		} else if (host) {
