@@ -17,7 +17,7 @@
 #endif
 
 #undef _WIN32_WINNT
-#define _WIN32_WINNT  0x0500
+#define _WIN32_WINNT  0x0602
 #include <windows.h>
 
 #include "Scintilla.h"
@@ -146,6 +146,15 @@ gui_string StringFromUTF8(const char *s) {
 	return gui_string(&vgc[0], outLen);
 }
 
+gui_string StringFromUTF8(const std::string &s) {
+	size_t sLen = s.length();
+	size_t wideLen = UTF16Length(s.c_str(), static_cast<int>(sLen));
+	std::vector<gui_char> vgc(wideLen + 1);
+	size_t outLen = UTF16FromUTF8(s.c_str(), sLen, &vgc[0], wideLen);
+	vgc[outLen] = 0;
+	return gui_string(&vgc[0], outLen);
+}
+
 std::string UTF8FromString(const gui_string &s) {
 	size_t sLen = s.size();
 	size_t narrowLen = UTF8Length(s.c_str(), sLen);
@@ -236,7 +245,7 @@ void Menu::Destroy() {
 
 void Menu::Show(Point pt, Window &w) {
 	::TrackPopupMenu(reinterpret_cast<HMENU>(mid),
-		0, pt.x - 4, pt.y, 0,
+		TPM_RIGHTBUTTON, pt.x - 4, pt.y, 0,
 		reinterpret_cast<HWND>(w.GetID()), NULL);
 	Destroy();
 }
