@@ -54,38 +54,32 @@ long SciTEKeys::ParseKeyCode(const char *mnemonic) {
 	int keyval = -1;
 
 	if (mnemonic && *mnemonic) {
-		SString sKey = mnemonic;
+		std::string sKey = mnemonic;
 
-		if (sKey.contains("Ctrl+")) {
+		if (RemoveStringOnce(sKey, "Ctrl+"))
 			modsInKey |= SCMOD_CTRL;
-			sKey.remove("Ctrl+");
-		}
-		if (sKey.contains("Shift+")) {
+		if (RemoveStringOnce(sKey, "Shift+"))
 			modsInKey |= SCMOD_SHIFT;
-			sKey.remove("Shift+");
-		}
-		if (sKey.contains("Alt+")) {
+		if (RemoveStringOnce(sKey, "Alt+"))
 			modsInKey |= SCMOD_ALT;
-			sKey.remove("Alt+");
-		}
 
 		if (sKey.length() == 1) {
 			keyval = VkKeyScan(sKey[0]) & 0xFF;
 		} else if (sKey.length() > 1) {
 			if ((sKey[0] == 'F') && (isdigit(sKey[1]))) {
-				sKey.remove("F");
-				int fkeyNum = sKey.value();
+				sKey.erase(0, 1);
+				int fkeyNum = atoi(sKey.c_str());
 				if (fkeyNum >= 1 && fkeyNum <= 12)
 					keyval = fkeyNum - 1 + VK_F1;
 			} else if ((sKey[0] == 'V') && (isdigit(sKey[1]))) {
-				sKey.remove("V");
-				int vkey = sKey.value();
+				sKey.erase(0, 1);
+				int vkey = atoi(sKey.c_str());
 				if (vkey > 0 && vkey <= 0x7FFF)
 					keyval = vkey;
-			} else if (sKey.search("Keypad") == 0) {
-				sKey.remove("Keypad");
+			} else if (sKey.find("Keypad") == 0) {
+				sKey.erase(0, strlen("Keypad"));
 				if (isdigit(sKey[0])) {
-					int keyNum = sKey.value();
+					int keyNum = atoi(sKey.c_str());
 					if (keyNum >= 0 && keyNum <= 9)
 						keyval = keyNum + VK_NUMPAD0;
 				} else if (sKey == "Plus") {
@@ -206,6 +200,9 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 	pathAbbreviations = GetAbbrevPropertiesFileName();
 
 	ReadGlobalPropFile();
+
+	SetScaleFactor(0);
+
 	tbLarge = props.GetInt("toolbar.large");
 	/// Need to copy properties to variables before setting up window
 	SetPropertiesInitial();
@@ -303,17 +300,25 @@ static int CodePageFromName(const std::string &encodingName) {
 }
 
 static std::string StringEncode(std::wstring s, int codePage) {
-	int cchMulti = ::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.size()), NULL, 0, NULL, NULL);
-	std::string sMulti(cchMulti, 0);
-	::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.size()), &sMulti[0], cchMulti, NULL, NULL);
-	return sMulti;
+	if (s.length()) {
+		int cchMulti = ::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0, NULL, NULL);
+		std::string sMulti(cchMulti, 0);
+		::WideCharToMultiByte(codePage, 0, s.c_str(), static_cast<int>(s.size()), &sMulti[0], cchMulti, NULL, NULL);
+		return sMulti;
+	} else {
+		return std::string();
+	}
 }
 
 static std::wstring StringDecode(std::string s, int codePage) {
-	int cchWide = ::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0);
-	std::wstring sWide(cchWide, 0);
-	::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), &sWide[0], cchWide);
-	return sWide;
+	if (s.length()) {
+		int cchWide = ::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), NULL, 0);
+		std::wstring sWide(cchWide, 0);
+		::MultiByteToWideChar(codePage, 0, s.c_str(), static_cast<int>(s.length()), &sWide[0], cchWide);
+		return sWide;
+	} else {
+		return std::wstring();
+	}
 }
 
 // Convert to UTF-8
@@ -357,6 +362,16 @@ void SciTEWin::GetWindowPosition(int *left, int *top, int *width, int *height, i
 	*maximize = (winPlace.showCmd == SW_MAXIMIZE) ? 1 : 0;
 }
 
+void SciTEWin::SetScaleFactor(int scale) {
+	if (scale == 0) {
+		HDC hdcMeasure = ::CreateCompatibleDC(NULL);
+		scale = ::GetDeviceCaps(hdcMeasure, LOGPIXELSX) * 100 / 96;
+		::DeleteDC(hdcMeasure);
+	}
+	std::string sScale = StdStringFromInteger(scale);
+	propsEmbed.Set("ScaleFactor", sScale.c_str());
+}
+
 // Allow UTF-8 file names and command lines to be used in calls to io.open and io.popen in Lua scripts.
 // The scite_lua_win.h header redirects fopen and _popen to these functions.
 
@@ -771,21 +786,11 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 		codePageOutput = CodePageFromCharSet(characterSet, codePageOutput);
 	}
 
-	SECURITY_ATTRIBUTES sa = {sizeof(SECURITY_ATTRIBUTES), 0, 0};
+	SECURITY_ATTRIBUTES sa = {sizeof(SECURITY_ATTRIBUTES), NULL, TRUE};
 	OutputAppendStringSynchronised(">");
-	OutputAppendEncodedStringSynchronised(GUI::StringFromUTF8(jobToRun.command.c_str()), codePageOutput);
+	OutputAppendEncodedStringSynchronised(GUI::StringFromUTF8(jobToRun.command), codePageOutput);
 	OutputAppendStringSynchronised("\n");
 
-	sa.bInheritHandle = TRUE;
-	sa.lpSecurityDescriptor = NULL;
-
-	SECURITY_DESCRIPTOR sd;
-	// Make a real security thing to allow inheriting handles
-	::InitializeSecurityDescriptor(&sd, SECURITY_DESCRIPTOR_REVISION);
-	::SetSecurityDescriptorDacl(&sd, TRUE, NULL, FALSE);
-	sa.nLength = sizeof(SECURITY_ATTRIBUTES);
-	sa.lpSecurityDescriptor = &sd;
-
 	HANDLE hPipeWrite = NULL;
 	HANDLE hPipeRead = NULL;
 	// Create pipe for output redirection
@@ -826,7 +831,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 	bool running = ::CreateProcessW(
 			  NULL,
-			  const_cast<wchar_t *>(GUI::StringFromUTF8(jobToRun.command.c_str()).c_str()),
+			  const_cast<wchar_t *>(GUI::StringFromUTF8(jobToRun.command).c_str()),
 			  NULL, NULL,
 			  TRUE, CREATE_NEW_PROCESS_GROUP,
 			  NULL,
@@ -837,12 +842,12 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 	// if jobCLI "System can't find" - try calling with command processor
 	if ((!running) && (jobToRun.jobType == jobCLI) && (::GetLastError() == ERROR_FILE_NOT_FOUND)) {
 
-		SString runComLine = "cmd.exe /c ";
-		runComLine = runComLine.append(jobToRun.command.c_str());
+		std::string runComLine = "cmd.exe /c ";
+		runComLine = runComLine.append(jobToRun.command);
 
 		running = ::CreateProcessW(
 			  NULL,
-			  const_cast<wchar_t*>(GUI::StringFromUTF8(runComLine.c_str()).c_str()),
+			  const_cast<wchar_t*>(GUI::StringFromUTF8(runComLine).c_str()),
 			  NULL, NULL,
 			  TRUE, CREATE_NEW_PROCESS_GROUP,
 			  NULL,
@@ -856,7 +861,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 		bool cancelled = false;
 
-		SString repSelBuf;
+		std::string repSelBuf;
 
 		size_t totalBytesToWrite = 0;
 		if (jobToRun.flags & jobHasInput) {
@@ -864,8 +869,8 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 		}
 
 		if (totalBytesToWrite > 0 && !(jobToRun.flags & jobQuiet)) {
-			SString input = jobToRun.input;
-			input.substitute("\n", "\n>> ");
+			std::string input = jobToRun.input;
+			Substitute(input, "\n", "\n>> ");
 
 			OutputAppendStringSynchronised(">> ");
 			OutputAppendStringSynchronised(input.c_str());
@@ -886,10 +891,10 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 
 			DWORD bytesRead = 0;
 			DWORD bytesAvail = 0;
-			char buffer[16384];
+			std::vector<char> buffer(16*1024);
 
-			if (!::PeekNamedPipe(hPipeRead, buffer,
-					     sizeof(buffer), &bytesRead, &bytesAvail, NULL)) {
+			if (!::PeekNamedPipe(hPipeRead, &buffer[0],
+					     static_cast<DWORD>(buffer.size()), &bytesRead, &bytesAvail, NULL)) {
 				bytesAvail = 0;
 			}
 
@@ -898,8 +903,8 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				// with reads, so that our hRead buffer will not be overrun with results.
 
 				size_t bytesToWrite;
-				int eol_pos = jobToRun.input.search("\n", writingPosition);
-				if (eol_pos == -1) {
+				const size_t eol_pos = jobToRun.input.find("\n", writingPosition);
+				if (eol_pos == std::string::npos) {
 					bytesToWrite = totalBytesToWrite - writingPosition;
 				} else {
 					bytesToWrite = eol_pos + 1 - writingPosition;
@@ -935,13 +940,13 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 				}
 
 			} else if (bytesAvail > 0) {
-				int bTest = ::ReadFile(hPipeRead, buffer,
-						       sizeof(buffer), &bytesRead, NULL);
+				int bTest = ::ReadFile(hPipeRead, &buffer[0],
+						       static_cast<DWORD>(buffer.size()), &bytesRead, NULL);
 
 				if (bTest && bytesRead) {
 
 					if (jobToRun.flags & jobRepSelMask) {
-						repSelBuf.append(buffer, bytesRead);
+						repSelBuf.append(&buffer[0], bytesRead);
 					}
 
 					if (!(jobToRun.flags & jobQuiet)) {
@@ -950,7 +955,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun) {
 							cmdWorker.seenOutput = true;
 						}
 						// Display the data
-						OutputAppendStringSynchronised(buffer, bytesRead);
+						OutputAppendStringSynchronised(&buffer[0], bytesRead);
 					}
 
 					::UpdateWindow(MainHWND());
@@ -1056,38 +1061,33 @@ void SciTEWin::ProcessExecute() {
 	PostOnMainThread(WORK_EXECUTE, &cmdWorker);
 }
 
-void SciTEWin::ShellExec(const SString &cmd, const char *dir) {
-	char *mycmd;
-
+void SciTEWin::ShellExec(const std::string &cmd, const char *dir) {
 	// guess if cmd is an executable, if this succeeds it can
 	// contain spaces without enclosing it with "
-	std::string cmdLower = cmd.c_str();
+	std::string cmdLower = cmd;
 	LowerCaseAZ(cmdLower);
-	char *mycmdcopy = StringDup(cmdLower.c_str());
+	const char *mycmdlowered = cmdLower.c_str();
 
+	const char *s = strstr(mycmdlowered, ".exe");
+	if (s == NULL)
+		s = strstr(mycmdlowered, ".cmd");
+	if (s == NULL)
+		s = strstr(mycmdlowered, ".bat");
+	if (s == NULL)
+		s = strstr(mycmdlowered, ".com");
+	std::vector<char> cmdcopy(cmd.c_str(), cmd.c_str() + cmd.length() + 1);
+	char *mycmdcopy = &cmdcopy[0];
+	char *mycmd;
 	char *mycmd_end = NULL;
-	char *myparams = NULL;
-
-	char *s = strstr(mycmdcopy, ".exe");
-	if (s == NULL)
-		s = strstr(mycmdcopy, ".cmd");
-	if (s == NULL)
-		s = strstr(mycmdcopy, ".bat");
-	if (s == NULL)
-		s = strstr(mycmdcopy, ".com");
 	if ((s != NULL) && ((*(s + 4) == '\0') || (*(s + 4) == ' '))) {
-		ptrdiff_t len_mycmd = s - mycmdcopy + 4;
-		delete []mycmdcopy;
-		mycmdcopy = StringDup(cmd.c_str());
+		ptrdiff_t len_mycmd = s - mycmdlowered + 4;
 		mycmd = mycmdcopy;
 		mycmd_end = mycmdcopy + len_mycmd;
 	} else {
-		delete []mycmdcopy;
-		mycmdcopy = StringDup(cmd.c_str());
 		if (*mycmdcopy != '"') {
 			// get next space to separate cmd and parameters
-			mycmd_end = strchr(mycmdcopy, ' ');
 			mycmd = mycmdcopy;
+			mycmd_end = strchr(mycmdcopy, ' ');
 		} else {
 			// the cmd is surrounded by ", so it can contain spaces, but we must
 			// strip the " for ShellExec
@@ -1100,6 +1100,7 @@ void SciTEWin::ShellExec(const SString &cmd, const char *dir) {
 		}
 	}
 
+	std::string myparams;
 	if ((mycmd_end != NULL) && (*mycmd_end != '\0')) {
 		*mycmd_end = '\0';
 		// test for remaining params after cmd, they may be surrounded by " but
@@ -1127,22 +1128,19 @@ void SciTEWin::ShellExec(const SString &cmd, const char *dir) {
 
 	if (::ShellExecuteEx(&exec)) {
 		// it worked!
-		delete []mycmdcopy;
 		return;
 	}
 	DWORD rc = GetLastError();
 
-	SString errormsg("Error while launching:\n\"");
+	std::string errormsg("Error while launching:\n\"");
 	errormsg += mycmdcopy;
-	if (myparams != NULL) {
+	if (myparams.length()) {
 		errormsg += "\" with Params:\n\"";
 		errormsg += myparams;
 	}
 	errormsg += "\"\n";
-	GUI::gui_string sErrorMsg = GUI::StringFromUTF8(errormsg.c_str()) + GetErrorMessage(rc);
+	GUI::gui_string sErrorMsg = GUI::StringFromUTF8(errormsg) + GetErrorMessage(rc);
 	WindowMessageBox(wSciTE, sErrorMsg, mbsOK);
-
-	delete []mycmdcopy;
 }
 
 void SciTEWin::Execute() {
@@ -1197,7 +1195,8 @@ void SciTEWin::StopExecute() {
 		if (!GenerateConsoleCtrlEvent(CTRL_BREAK_EVENT, subProcessGroupId)) {
 			LONG errCode = GetLastError();
 			OutputAppendStringSynchronised("\n>BREAK Failed ");
-			OutputAppendStringSynchronised(SString(errCode).c_str());
+			std::string sError = StdStringFromInteger(errCode);
+			OutputAppendStringSynchronised(sError.c_str());
 			OutputAppendStringSynchronised("\n");
 		}
 		Sleep(100L);
@@ -1207,13 +1206,13 @@ void SciTEWin::StopExecute() {
 	jobQueue.SetCancelFlag(1);
 }
 
-void SciTEWin::AddCommand(const SString &cmd, const SString &dir, JobSubsystem jobType, const SString &input, int flags) {
+void SciTEWin::AddCommand(const std::string &cmd, const std::string &dir, JobSubsystem jobType, const std::string &input, int flags) {
 	if (cmd.length()) {
 		if ((jobType == jobShell) && ((flags & jobForceQueue) == 0)) {
-			SString pCmd = cmd;
+			std::string pCmd = cmd;
 			parameterisedCommand = "";
 			if (pCmd[0] == '*') {
-				pCmd.remove(0, 1);
+				pCmd.erase(0, 1);
 				parameterisedCommand = pCmd;
 				if (!ParametersDialog(true)) {
 					return;
@@ -1221,7 +1220,7 @@ void SciTEWin::AddCommand(const SString &cmd, const SString &dir, JobSubsystem j
 			} else {
 				ParamGrab();
 			}
-			pCmd = props.Expand(pCmd.c_str());
+			pCmd = props.Expand(pCmd);
 			ShellExec(pCmd, dir.c_str());
 		} else {
 			SciTEBase::AddCommand(cmd, dir, jobType, input, flags);
@@ -1326,9 +1325,9 @@ void SciTEWin::CreateUI() {
 		RestorePosition();
 
 	LocaliseMenus();
-	SString pageSetup = props.Get("print.margins");
+	std::string pageSetup = props.GetString("print.margins");
 	char val[32];
-	char *ps = StringDup(pageSetup.c_str());
+	const char *ps = pageSetup.c_str();
 	const char *next = GetNextPropItem(ps, val, 32);
 	pagesetupMargin.left = atol(val);
 	next = GetNextPropItem(next, val, 32);
@@ -1337,7 +1336,6 @@ void SciTEWin::CreateUI() {
 	pagesetupMargin.top = atol(val);
 	GetNextPropItem(next, val, 32);
 	pagesetupMargin.bottom = atol(val);
-	delete []ps;
 
 	UIAvailable();
 }
@@ -1375,7 +1373,11 @@ GUI::gui_string SciTEWin::ProcessArgs(const GUI::gui_char *cmdLine) {
 			args += GUI_TEXT("\n");
 		args += arg;
 		startArg = endArg;	// On a space or a double-quote, or on the end of the command line
-		if (*startArg) {
+		if (*startArg == '"') {	// Closing double-quote
+			startArg++;	// Consume the double-quote
+		}
+		while (IsASpace(*startArg)) {
+			// Consume spaces between arguments
 			startArg++;
 		}
 	}
@@ -1555,7 +1557,7 @@ bool SciTEWin::PreOpenCheck(const GUI::gui_char *arg) {
 	if (fileattributes != (DWORD) -1) {	// arg is an existing directory or filename
 		// if the command line argument is a directory, use OpenDialog()
 		if (fileattributes & FILE_ATTRIBUTE_DIRECTORY) {
-			OpenDialog(fpArg, GUI::StringFromUTF8(props.GetExpanded("open.filter").c_str()).c_str());
+			OpenDialog(fpArg, GUI::StringFromUTF8(props.GetExpandedString("open.filter")).c_str());
 			isHandled = true;
 		}
 	} else if (nbuffers > 1 && (hFFile = ::FindFirstFile(arg, &ffile)) != INVALID_HANDLE_VALUE) {
@@ -1587,9 +1589,9 @@ bool SciTEWin::PreOpenCheck(const GUI::gui_char *arg) {
 			OpenDialog(fpArg.Directory(), wildcard.c_str());
 		} else if (!fpArg.Extension().IsSet()) {
 			// if the filename has no extension, try to match a file with list of standard extensions
-			SString extensions = props.GetExpanded("source.default.extensions");
+			std::string extensions = props.GetExpandedString("source.default.extensions");
 			if (extensions.length()) {
-				extensions.substitute('|', '\0');
+				std::replace(extensions.begin(), extensions.end(), '|', '\0');
 				size_t start = 0;
 				while (start < extensions.length()) {
 					GUI::gui_string filterName = GUI::StringFromUTF8(extensions.c_str() + start);
@@ -1927,6 +1929,11 @@ LRESULT SciTEWin::WndProc(UINT iMessage, WPARAM wParam, LPARAM lParam) {
 			wOutput.Call(WM_SYSCOLORCHANGE, wParam, lParam);
 			break;
 
+		case WM_DPICHANGED:
+			SetScaleFactor(LOWORD(wParam) * 100 / 96);
+			ReadProperties();
+			return ::DefWindowProcW(MainHWND(), iMessage, wParam, lParam);
+
 		case WM_ACTIVATEAPP:
 			wEditor.Call(SCI_HIDESELECTION, !wParam);
 			// Do not want to display dialog yet as may be in middle of system mouse capture
@@ -2078,8 +2085,8 @@ std::string SciTEWin::EncodeString(const std::string &s) {
 }
 
 // Convert String from doc encoding to UTF-8
-SString SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd) {
-	SString s = SciTEBase::GetRangeInUIEncoding(win, selStart, selEnd);
+std::string SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart, int selEnd) {
+	std::string s = SciTEBase::GetRangeInUIEncoding(win, selStart, selEnd);
 
 	UINT codePageDocument = wEditor.Call(SCI_GETCODEPAGE);
 
@@ -2087,7 +2094,7 @@ SString SciTEWin::GetRangeInUIEncoding(GUI::ScintillaWindow &win, int selStart,
 		codePageDocument = CodePageFromCharSet(characterSet, codePageDocument);
 		std::wstring sWide = StringDecode(std::string(s.c_str(), s.length()), codePageDocument);
 		std::string sMulti = StringEncode(sWide, CP_UTF8);
-		return SString(sMulti.c_str(), 0, sMulti.length());
+		return std::string(sMulti.c_str(), 0, sMulti.length());
 	}
 	return s;
 }
