@@ -36,7 +36,7 @@ void SciTEWin::SetFileProperties(
 		ps.Set("FileDate", temp);
 
 		DWORD attr = ::GetFileAttributesW(filePath.AsInternal());
-		SString fa;
+		std::string fa;
 		if (attr & FILE_ATTRIBUTE_READONLY) {
 			fa += "R";
 		}
@@ -93,7 +93,7 @@ void SciTEWin::RemoveAllTabs() {
 /**
  * Manage Windows specific notifications.
  */
-void SciTEWin::Notify(SCNotification *notification) {
+void SciTEWin::Notify(const SCNotification *notification) {
 	switch (notification->nmhdr.code) {
 	case TCN_SELCHANGE:
 		// Change of tab
@@ -136,15 +136,12 @@ void SciTEWin::Notify(SCNotification *notification) {
 			bool bAddSeparator = false;
 			for (int item = 0; item < toolMax; item++) {
 				int itemID = IDM_TOOLS + item;
-				SString prefix = "command.name.";
-				prefix += SString(item);
+				std::string prefix = "command.name.";
+				prefix += StdStringFromInteger(item);
 				prefix += ".";
-				SString commandName = props.GetNewExpand(prefix.c_str(), filePath.AsUTF8().c_str());
+				std::string commandName = props.GetNewExpandString(prefix.c_str(), filePath.AsUTF8().c_str());
 				if (commandName.length()) {
-					SString sMenuItem = commandName;
-					SString sMnemonic = "Ctrl+";
-					sMnemonic += SString(item);
-					AddToPopUp(sMenuItem.c_str(), itemID, true);
+					AddToPopUp(commandName.c_str(), itemID, true);
 					bAddSeparator = true;
 				}
 			}
@@ -1055,7 +1052,7 @@ void SciTEWin::Creation() {
 	bands.push_back(Band(true, heightStatus, false, wStatusBar));
 
 #ifndef NO_LUA
-		if (props.GetExpanded("ext.lua.startup.script").length() == 0)
+		if (props.GetExpandedString("ext.lua.startup.script").length() == 0)
 			DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
 #else
 		DestroyMenuItem(menuOptions,IDM_OPENLUAEXTERNALFILE);
