@@ -83,38 +83,36 @@ static SciTEWin *Caller(HWND hDlg, UINT message, LPARAM lParam) {
 }
 
 void SciTEWin::WarnUser(int warnID) {
-	SString warning;
-	char *warn;
+	std::string warning;
 	char flashDuration[10], sound[_MAX_PATH], soundDuration[10];
 
 	switch (warnID) {
 	case warnFindWrapped:
-		warning = props.Get("warning.findwrapped");
+		warning = props.GetString("warning.findwrapped");
 		break;
 	case warnNotFound:
-		warning = props.Get("warning.notfound");
+		warning = props.GetString("warning.notfound");
 		break;
 	case warnWrongFile:
-		warning = props.Get("warning.wrongfile");
+		warning = props.GetString("warning.wrongfile");
 		break;
 	case warnExecuteOK:
-		warning = props.Get("warning.executeok");
+		warning = props.GetString("warning.executeok");
 		break;
 	case warnExecuteKO:
-		warning = props.Get("warning.executeko");
+		warning = props.GetString("warning.executeko");
 		break;
 	case warnNoOtherBookmark:
-		warning = props.Get("warning.nootherbookmark");
+		warning = props.GetString("warning.nootherbookmark");
 		break;
 	default:
 		warning = "";
 		break;
 	}
-	warn = StringDup(warning.c_str());
+	const char *warn = warning.c_str();
 	const char *next = GetNextPropItem(warn, flashDuration, 10);
 	next = GetNextPropItem(next, sound, _MAX_PATH);
 	GetNextPropItem(next, soundDuration, 10);
-	delete []warn;
 
 	int flashLen = atoi(flashDuration);
 	if (flashLen) {
@@ -317,7 +315,7 @@ FilePath SciTEWin::ChooseSaveName(FilePath directory, const char *title, const G
 
 bool SciTEWin::SaveAsDialog() {
 	GUI::gui_string saveFilter = DialogFilterFromProperty(
-		GUI::StringFromUTF8(props.GetExpanded("save.filter").c_str()).c_str());
+		GUI::StringFromUTF8(props.GetExpandedString("save.filter")).c_str());
 	FilePath path = ChooseSaveName(filePath.Directory(), "Save File", saveFilter.c_str());
 	if (path.IsSet()) {
 		return SaveIfNotOpen(path, false);
@@ -549,13 +547,12 @@ void SciTEWin::Print(
 	// Convert page size to logical units and we're done!
 	DPtoLP(hdc, (LPPOINT) &ptPage, 1);
 
-	SString headerFormat = props.Get("print.header.format");
-	SString footerFormat = props.Get("print.footer.format");
+	std::string headerFormat = props.GetString("print.header.format");
+	std::string footerFormat = props.GetString("print.footer.format");
 
 	TEXTMETRIC tm;
-	SString headerOrFooter;	// Usually the path, date and page number
 
-	SString headerStyle = props.Get("print.header.style");
+	std::string headerStyle = props.GetString("print.header.style");
 	StyleDefinition sdHeader(headerStyle.c_str());
 
 	int headerLineHeight = ::MulDiv(
@@ -573,7 +570,7 @@ void SciTEWin::Print(
 	::GetTextMetrics(hdc, &tm);
 	headerLineHeight = tm.tmHeight + tm.tmExternalLeading;
 
-	SString footerStyle = props.Get("print.footer.style");
+	std::string footerStyle = props.GetString("print.footer.style");
 	StyleDefinition sdFooter(footerStyle.c_str());
 
 	int footerLineHeight = ::MulDiv(
@@ -661,7 +658,7 @@ void SciTEWin::Print(
 			::StartPage(hdc);
 
 			if (headerFormat.size()) {
-				GUI::gui_string sHeader = GUI::StringFromUTF8(propsPrint.GetExpanded("print.header.format").c_str());
+				GUI::gui_string sHeader = GUI::StringFromUTF8(propsPrint.GetExpandedString("print.header.format"));
 				::SetTextColor(hdc, sdHeader.ForeAsLong());
 				::SetBkColor(hdc, sdHeader.BackAsLong());
 				::SelectObject(hdc, fontHeader);
@@ -691,7 +688,7 @@ void SciTEWin::Print(
 
 		if (printPage) {
 			if (footerFormat.size()) {
-				GUI::gui_string sFooter = GUI::StringFromUTF8(propsPrint.GetExpanded("print.footer.format").c_str());
+				GUI::gui_string sFooter = GUI::StringFromUTF8(propsPrint.GetExpandedString("print.footer.format"));
 				::SetTextColor(hdc, sdFooter.ForeAsLong());
 				::SetBkColor(hdc, sdFooter.BackAsLong());
 				::SelectObject(hdc, fontFooter);
@@ -766,7 +763,7 @@ class Dialog {
 	HWND hDlg;
 public:
 
-	Dialog(HWND hDlg_) : hDlg(hDlg_) {
+	explicit Dialog(HWND hDlg_) : hDlg(hDlg_) {
 	}
 
 	HWND Item(int id) {
@@ -794,13 +791,12 @@ public:
 
 	// Handle Unicode controls (assume strings to be UTF-8 on Windows NT)
 
-	SString ItemTextU(int id) {
-		SString s = GUI::UTF8FromString(ItemTextG(id).c_str()).c_str();
-		return s;
+	std::string ItemTextU(int id) {
+		return GUI::UTF8FromString(ItemTextG(id));
 	}
 
 	void SetItemTextU(int id, const std::string &s) {
-		SetItemText(id, GUI::StringFromUTF8(s.c_str()).c_str());
+		SetItemText(id, GUI::StringFromUTF8(s).c_str());
 	}
 
 	void SetCheck(int id, bool value) {
@@ -816,7 +812,7 @@ public:
 		HWND combo = Item(id);
 		::SendMessage(combo, CB_RESETCONTENT, 0, 0);
 		for (int i = 0; i < mem.Length(); i++) {
-			GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i).c_str());
+			GUI::gui_string gs = GUI::StringFromUTF8(mem.At(i));
 			::SendMessageW(combo, CB_ADDSTRING, 0,
 				       reinterpret_cast<LPARAM>(gs.c_str()));
 		}
@@ -1093,10 +1089,6 @@ void SciTEWin::FindIncrement() {
 	}
 }
 
-bool SciTEWin::FindReplaceAdvanced() {
-	return props.GetInt("find.replace.advanced");
-}
-
 void SciTEWin::Find() {
 	if (wFindReplace.Created()) {
 		if (!replacing) {
@@ -1149,31 +1141,31 @@ static int __stdcall BrowseCallbackProc(HWND hwnd, UINT uMsg, LPARAM, LPARAM pDa
 void SciTEWin::PerformGrep() {
 	SelectionIntoProperties();
 
-	SString findInput;
+	std::string findInput;
 	long flags = 0;
-	if (props.Get("find.input").length()) {
-		findInput = props.GetNewExpand("find.input");
+	if (props.GetString("find.input").length()) {
+		findInput = props.GetNewExpandString("find.input");
 		flags += jobHasInput;
 	}
 
-	SString findCommand = props.GetNewExpand("find.command");
+	std::string findCommand = props.GetNewExpandString("find.command");
 	if (findCommand == "") {
 		// Call InternalGrep in a new thread
 		// searchParams is "(w|~)(c|~)(d|~)(b|~)\0files\0text"
 		// A "w" indicates whole word, "c" case sensitive, "d" dot directories, "b" binary files
-		SString searchParams;
+		std::string searchParams;
 		searchParams.append(wholeWord ? "w" : "~");
 		searchParams.append(matchCase ? "c" : "~");
 		searchParams.append(props.GetInt("find.in.dot") ? "d" : "~");
 		searchParams.append(props.GetInt("find.in.binary") ? "b" : "~");
 		searchParams.append("\0", 1);
-		searchParams.append(props.Get("find.files").c_str());
+		searchParams.append(props.GetString("find.files"));
 		searchParams.append("\0", 1);
-		searchParams.append(props.Get("find.what").c_str());
-		AddCommand(searchParams, props.Get("find.directory"), jobGrep, findInput, flags);
+		searchParams.append(props.GetString("find.what"));
+		AddCommand(searchParams, props.GetString("find.directory"), jobGrep, findInput, flags);
 	} else {
 		AddCommand(findCommand,
-			   props.Get("find.directory"),
+			   props.GetString("find.directory"),
 			   jobCLI, findInput, flags);
 	}
 	if (jobQueue.HasCommandToRun()) {
@@ -1199,7 +1191,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		FillCombos(dlg);
 		dlg.SetItemTextU(IDFINDWHAT, props.GetString("find.what"));
 		dlg.SetItemTextU(IDDIRECTORY, props.GetString("find.directory"));
-		if (props.GetNewExpand("find.command") == "") {
+		if (props.GetNewExpandString("find.command") == "") {
 			// Empty means use internal that can respond to flags
 			dlg.SetCheck(IDWHOLEWORD, wholeWord);
 			dlg.SetCheck(IDMATCHCASE, matchCase);
@@ -1225,17 +1217,17 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 				WindowMessageBox(wFindInFiles, msgBuf);
 				return FALSE;
 			}
-			findWhat = dlg.ItemTextU(IDFINDWHAT).string();
+			findWhat = dlg.ItemTextU(IDFINDWHAT);
 			props.Set("find.what", findWhat.c_str());
 			InsertFindInMemory();
 
-			SString files = dlg.ItemTextU(IDFILES);
+			std::string files = dlg.ItemTextU(IDFILES);
 			props.Set("find.files", files.c_str());
-			memFiles.Insert(files.c_str());
+			memFiles.Insert(files);
 
-			SString directory = dlg.ItemTextU(IDDIRECTORY);
+			std::string directory = dlg.ItemTextU(IDDIRECTORY);
 			props.Set("find.directory", directory.c_str());
-			memDirectory.Insert(directory.c_str());
+			memDirectory.Insert(directory);
 
 			wholeWord = dlg.Checked(IDWHOLEWORD);
 			matchCase = dlg.Checked(IDMATCHCASE);
@@ -1322,7 +1314,7 @@ void SciTEWin::FindInFiles() {
 	}
 	props.Set("find.what", findWhat.c_str());
 
-	SString directory = props.Get("find.in.directory");
+	std::string directory = props.GetString("find.in.directory");
 	if (directory.length()) {
 		props.Set("find.directory", directory.c_str());
 	} else {
@@ -1571,7 +1563,7 @@ void SciTEWin::ParamGrab() {
 		Dialog dlg(hDlg);
 		for (int param = 0; param < maxParam; param++) {
 			std::string paramVal = GUI::UTF8FromString(dlg.ItemTextG(IDPARAMSTART + param));
-			SString paramText(param + 1);
+			std::string paramText = StdStringFromInteger(param + 1);
 			props.Set(paramText.c_str(), paramVal.c_str());
 		}
 		UpdateStatusBar(true);
@@ -1586,13 +1578,13 @@ BOOL SciTEWin::ParametersMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			wParameters = hDlg;
 			Dialog dlg(hDlg);
 			if (modalParameters) {
-				GUI::gui_string sCommand = GUI::StringFromUTF8(parameterisedCommand.c_str());
+				GUI::gui_string sCommand = GUI::StringFromUTF8(parameterisedCommand);
 				dlg.SetItemText(IDCMD, sCommand.c_str());
 			}
 			for (int param = 0; param < maxParam; param++) {
-				SString paramText(param + 1);
-				SString paramTextVal = props.Get(paramText.c_str());
-				GUI::gui_string sVal = GUI::StringFromUTF8(paramTextVal.c_str());
+				std::string paramText = StdStringFromInteger(param + 1);
+				std::string paramTextVal = props.GetString(paramText.c_str());
+				GUI::gui_string sVal = GUI::StringFromUTF8(paramTextVal);
 				dlg.SetItemText(IDPARAMSTART + param, sVal.c_str());
 			}
 		}
@@ -1673,12 +1665,12 @@ SciTEBase::MessageBoxChoice SciTEWin::WindowMessageBox(GUI::Window &w, const GUI
 	}
 }
 
-void SciTEWin::FindMessageBox(const SString &msg, const std::string *findItem) {
+void SciTEWin::FindMessageBox(const std::string &msg, const std::string *findItem) {
 	if (findItem == 0) {
 		GUI::gui_string msgBuf = LocaliseMessage(msg.c_str());
 		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf);
 	} else {
-		GUI::gui_string findThing = GUI::StringFromUTF8(findItem->c_str());
+		GUI::gui_string findThing = GUI::StringFromUTF8(*findItem);
 		GUI::gui_string msgBuf = LocaliseMessage(msg.c_str(), findThing.c_str());
 		WindowMessageBox(wFindReplace.Created() ? wFindReplace : wSciTE, msgBuf);
 	}
