@@ -41,6 +41,7 @@ protected:
 	HFONT fontText;
 	HTHEME hTheme;
 	int scale;
+	int space;
 	bool capturedMouse;
 	SIZE closeSize;
 	enum stripCloseState { csNone, csOver, csClicked, csClickedOver } closeState;
@@ -49,7 +50,7 @@ protected:
 	int lineHeight;
 
 	GUI::Window CreateText(const char *text);
-	GUI::Window CreateButton(const char *text, int ident, bool check=false);
+	GUI::Window CreateButton(const char *text, size_t ident, bool check=false);
 	void Tab(bool forwards);
 	virtual void Creation();
 	virtual void Destruction();
@@ -72,12 +73,12 @@ protected:
 	virtual void ShowPopup();
 public:
 	bool visible;
-	Strip() : fontText(0), hTheme(0), scale(96), capturedMouse(false), closeState(csNone), entered(0), lineHeight(20), visible(false) {
+	Strip() : fontText(0), hTheme(0), scale(96), space(2), capturedMouse(false), closeState(csNone), entered(0), lineHeight(20), visible(false) {
 		closeSize.cx = 11;
 		closeSize.cy = 11;
 	}
 	virtual int Height() {
-		return lineHeight * Lines() + 1;
+		return lineHeight * Lines() + space - 1;
 	}
 };
 
