@@ -190,19 +190,14 @@ void UniqueInstance::SendCommands(const char *cmdLine) {
 	// (Restoring the cwd could be done,
 	// but keeping it to the last file opened can also
 	// be useful)
-	char cwdCmd[MAX_PATH + 7]; // 7 for "-cwd:" and 2x'"'
-	strcpy(cwdCmd, "\"-cwd:");
+	std::string cwdCmd("\"-cwd:");
 	FilePath cwd = FilePath::GetWorkingDirectory();
-	strncpy(cwdCmd + strlen(cwdCmd), cwd.AsUTF8().c_str(), MAX_PATH);
-	strcat(cwdCmd, "\"");
+	cwdCmd.append(cwd.AsUTF8().c_str());
+	cwdCmd.append("\"");
 	// Defeat the "\" mangling - convert "\" to "/"
-	for (char *temp = cwdCmd; *temp; temp++) {
-		if (*temp == '\\') {
-			*temp = '/';
-		}
-	}
-	cds.cbData = static_cast<DWORD>(strlen(cwdCmd) + 1);
-	cds.lpData = static_cast<void *>(cwdCmd);
+	std::replace(cwdCmd.begin(), cwdCmd.end(), '\\', '/');
+	cds.cbData = static_cast<DWORD>(cwdCmd.length() + 1);
+	cds.lpData = const_cast<void *>(static_cast<const void *>(cwdCmd.c_str()));
 	::SendMessage(hOtherWindow, WM_COPYDATA, 0,
 	              reinterpret_cast<LPARAM>(&cds));
 	// Now the command line itself.
